/***************************************************************************
                          crystalsymop.cpp  -  description
                             -------------------
    begin                : Mon Sept 28 07:53:00 CEST 2020
    copyright            : (C) 2020 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "crystalsymop.h"
#include "crystalatom.h"
#include <QRegularExpression>
#include <QDebug>

CrystalSymOp::CrystalSymOp(float _x11, float _x12, float _x13, float _x14,
                           float _x21, float _x22, float _x23, float _x24,
                           float _x31, float _x32, float _x33, float _x34)
    : _m11(_x11), _m12(_x12), _m13(_x13), _m14(_x14),
      _m21(_x21), _m22(_x22), _m23(_x23), _m24(_x24),
      _m31(_x31), _m32(_x32), _m33(_x33), _m34(_x34)
{

}

CrystalSymOp::CrystalSymOp(const CrystalSymOp &a)
    : _m11(a._m11), _m12(a._m12), _m13(a._m13), _m14(a._m14),
      _m21(a._m21), _m22(a._m22), _m23(a._m23), _m24(a._m24),
      _m31(a._m31), _m32(a._m32), _m33(a._m33), _m34(a._m34)
{

}

CrystalSymOp::CrystalSymOp(const QStringList &l)
{
    parseString(l);
}

CrystalSymOp::CrystalSymOp(const QString &s)
{
    parseString(s.split(QRegularExpression("[,\\s]")));
}

void CrystalSymOp::operator=(const CrystalSymOp &a)
{
    _m11 = a._m11;
    _m12 = a._m12;
    _m13 = a._m13;
    _m14 = a._m14;
    _m21 = a._m21;
    _m22 = a._m22;
    _m23 = a._m23;
    _m24 = a._m24;
    _m31 = a._m31;
    _m32 = a._m32;
    _m33 = a._m33;
    _m34 = a._m34;
}

bool CrystalSymOp::operator==(const CrystalSymOp &a) const
{
    if (!qFuzzyCompare(_m11, a._m11)) return false;
    if (!qFuzzyCompare(_m12, a._m12)) return false;
    if (!qFuzzyCompare(_m13, a._m13)) return false;
    if (!qFuzzyCompare(_m14, a._m14)) return false;
    if (!qFuzzyCompare(_m21, a._m21)) return false;
    if (!qFuzzyCompare(_m22, a._m22)) return false;
    if (!qFuzzyCompare(_m23, a._m23)) return false;
    if (!qFuzzyCompare(_m24, a._m24)) return false;
    if (!qFuzzyCompare(_m31, a._m31)) return false;
    if (!qFuzzyCompare(_m32, a._m32)) return false;
    if (!qFuzzyCompare(_m33, a._m33)) return false;
    if (!qFuzzyCompare(_m34, a._m34)) return false;
    return true;
}

bool CrystalSymOp::isEmpty() const
{
    if (!qFuzzyIsNull(_m11)) return false;
    if (!qFuzzyIsNull(_m12)) return false;
    if (!qFuzzyIsNull(_m13)) return false;
    if (!qFuzzyIsNull(_m14)) return false;
    if (!qFuzzyIsNull(_m21)) return false;
    if (!qFuzzyIsNull(_m22)) return false;
    if (!qFuzzyIsNull(_m23)) return false;
    if (!qFuzzyIsNull(_m24)) return false;
    if (!qFuzzyIsNull(_m31)) return false;
    if (!qFuzzyIsNull(_m32)) return false;
    if (!qFuzzyIsNull(_m33)) return false;
    if (!qFuzzyIsNull(_m34)) return false;
    return true;
}

void CrystalSymOp::setMatrix(float _x11, float _x12, float _x13, float _x14,
                             float _x21, float _x22, float _x23, float _x24,
                             float _x31, float _x32, float _x33, float _x34)
{
    _m11 = _x11;
    _m12 = _x12;
    _m13 = _x13;
    _m14 = _x14;
    _m21 = _x21;
    _m22 = _x22;
    _m23 = _x23;
    _m24 = _x24;
    _m31 = _x31;
    _m32 = _x32;
    _m33 = _x33;
    _m34 = _x34;
}

void CrystalSymOp::parseString(const QStringList &l)
{
    _m11 = 0.0;
    _m12 = 0.0;
    _m13 = 0.0;
    _m14 = 0.0;
    _m21 = 0.0;
    _m22 = 0.0;
    _m23 = 0.0;
    _m24 = 0.0;
    _m31 = 0.0;
    _m32 = 0.0;
    _m33 = 0.0;
    _m34 = 0.0;

    if (l.size() != 3) return;

    parseOperator(l.at(0), _m11, _m12, _m13, _m14);
    parseOperator(l.at(1), _m21, _m22, _m23, _m24);
    parseOperator(l.at(2), _m31, _m32, _m33, _m34);
}

void CrystalSymOp::parseOperator(const QString &o, float &_mx1, float &_mx2, float &_mx3, float &_mx4)
{
    QRegularExpression rx("([\\+-]?[x-z]|[\\+-]?[\\d\\/\\d]+)");
    QRegularExpressionMatchIterator ri = rx.globalMatch(o);

    while (ri.hasNext()) {
        QRegularExpression rxs("([\\+-]?)([^\\-]+)");
        QRegularExpressionMatch rm = rxs.match(ri.next().captured(1));

        bool sub = rm.captured(1) == "-";
        QString par = rm.captured(2);

        if      (par == "x") _mx1 += sub ? -1.0 : 1.0;
        else if (par == "y") _mx2 += sub ? -1.0 : 1.0;
        else if (par == "z") _mx3 += sub ? -1.0 : 1.0;

        else if (par == "1/2") _mx4 += sub ? -1.0/2.0 : 1.0/2.0;

        else if (par == "1/3") _mx4 += sub ? -1.0/3.0 : 1.0/3.0;
        else if (par == "2/3") _mx4 += sub ? -2.0/3.0 : 2.0/3.0;

        else if (par == "1/4") _mx4 += sub ? -1.0/4.0 : 1.0/4.0;
        else if (par == "2/4") _mx4 += sub ? -2.0/4.0 : 2.0/4.0;
        else if (par == "3/4") _mx4 += sub ? -3.0/4.0 : 3.0/4.0;

        else if (par == "1/6") _mx4 += sub ? -1.0/6.0 : 1.0/6.0;
        else if (par == "2/6") _mx4 += sub ? -2.0/6.0 : 2.0/6.0;
        else if (par == "3/6") _mx4 += sub ? -3.0/6.0 : 3.0/6.0;
        else if (par == "4/6") _mx4 += sub ? -4.0/6.0 : 4.0/6.0;
        else if (par == "5/6") _mx4 += sub ? -5.0/6.0 : 5.0/6.0;

        else if (par == "1/8") _mx4 += sub ? -1.0/8.0 : 1.0/8.0;
        else if (par == "2/8") _mx4 += sub ? -2.0/8.0 : 2.0/8.0;
        else if (par == "3/8") _mx4 += sub ? -3.0/8.0 : 3.0/8.0;
        else if (par == "4/8") _mx4 += sub ? -4.0/8.0 : 4.0/8.0;
        else if (par == "5/8") _mx4 += sub ? -5.0/8.0 : 5.0/8.0;
        else if (par == "6/8") _mx4 += sub ? -6.0/8.0 : 6.0/8.0;
        else if (par == "7/8") _mx4 += sub ? -7.0/8.0 : 7.0/8.0;

        else if (par == "1/12") _mx4 += sub ? -1.0/12.0 : 1.0/12.0;
        else if (par == "2/12") _mx4 += sub ? -2.0/12.0 : 2.0/12.0;
        else if (par == "3/12") _mx4 += sub ? -3.0/12.0 : 3.0/12.0;
        else if (par == "4/12") _mx4 += sub ? -4.0/12.0 : 4.0/12.0;
        else if (par == "5/12") _mx4 += sub ? -5.0/12.0 : 5.0/12.0;
        else if (par == "6/12") _mx4 += sub ? -6.0/12.0 : 6.0/12.0;
        else if (par == "7/12") _mx4 += sub ? -7.0/12.0 : 7.0/12.0;
        else if (par == "8/12") _mx4 += sub ? -8.0/12.0 : 8.0/12.0;
        else if (par == "9/12") _mx4 += sub ? -9.0/12.0 : 9.0/12.0;
        else if (par == "10/12") _mx4 += sub ? -10.0/12.0 : 10.0/12.0;
        else if (par == "11/12") _mx4 += sub ? -11.0/12.0 : 11.0/12.0;
    }
}

QList<CrystalSymOp> CrystalSymOp::translate(const QString &t)
{
    if (t.toUpper() == "I") {
        CrystalSymOp ci = *this;

        ci._m14 += 0.5;
        ci._m24 += 0.5;
        ci._m34 += 0.5;

        if (ci._m14 > 0.999) ci._m14 -= 1.0;
        if (ci._m24 > 0.999) ci._m24 -= 1.0;
        if (ci._m34 > 0.999) ci._m34 -= 1.0;

        return QList<CrystalSymOp>() << ci;
    }

    if (t.toUpper() == "F") {
        CrystalSymOp ca = *this;
        CrystalSymOp cb = *this;
        CrystalSymOp cc = *this;

        ca._m14 += 0.5;
        ca._m24 += 0.5;

        cb._m14 += 0.5;
        cb._m34 += 0.5;

        cc._m24 += 0.5;
        cc._m34 += 0.5;

        if (ca._m14 > 0.999) ca._m14 -= 1.0;
        if (ca._m24 > 0.999) ca._m24 -= 1.0;

        if (cb._m14 > 0.999) cb._m14 -= 1.0;
        if (cb._m34 > 0.999) cb._m34 -= 1.0;

        if (cc._m24 > 0.999) cc._m24 -= 1.0;
        if (cc._m34 > 0.999) cc._m34 -= 1.0;

        return QList<CrystalSymOp>() << ca << cb << cc;
    }

    if (t.toUpper() == "A") {
        CrystalSymOp ca = *this;

        ca._m24 += 0.5;
        ca._m34 += 0.5;

        if (ca._m24 > 0.999) ca._m24 -= 1.0;
        if (ca._m34 > 0.999) ca._m34 -= 1.0;

        return QList<CrystalSymOp>() << ca;
    }

    if (t.toUpper() == "B") {
        CrystalSymOp cb = *this;

        cb._m14 += 0.5;
        cb._m34 += 0.5;

        if (cb._m14 > 0.999) cb._m14 -= 1.0;
        if (cb._m34 > 0.999) cb._m34 -= 1.0;

        return QList<CrystalSymOp>() << cb;
    }

    if (t.toUpper() == "C") {
        CrystalSymOp cc = *this;

        cc._m14 += 0.5;
        cc._m24 += 0.5;

        if (cc._m14 > 0.999) cc._m14 -= 1.0;
        if (cc._m24 > 0.999) cc._m24 -= 1.0;

        return QList<CrystalSymOp>() << cc;
    }

    if (t.toUpper() == "R") {
        CrystalSymOp ca = *this;
        CrystalSymOp cb = *this;

        ca._m14 += 2.0/3.0;
        ca._m24 += 1.0/3.0;
        ca._m34 += 1.0/3.0;

        cb._m14 += 1.0/3.0;
        cb._m24 += 2.0/3.0;
        cb._m34 += 2.0/3.0;

        if (ca._m14 > 0.999) ca._m14 -= 1.0;
        if (ca._m24 > 0.999) ca._m24 -= 1.0;
        if (ca._m34 > 0.999) ca._m34 -= 1.0;

        if (cb._m14 > 0.999) cb._m14 -= 1.0;
        if (cb._m24 > 0.999) cb._m24 -= 1.0;
        if (cb._m34 > 0.999) cb._m34 -= 1.0;

        return QList<CrystalSymOp>() << ca << cb;
    }

    if (t.toUpper() == "D") {
        CrystalSymOp ca = *this;
        CrystalSymOp cb = *this;

        ca._m14 += 1.0/3.0;
        ca._m24 += 1.0/3.0;
        ca._m34 += 1.0/3.0;

        cb._m14 += 2.0/3.0;
        cb._m24 += 2.0/3.0;
        cb._m34 += 2.0/3.0;

        if (ca._m14 > 0.999) ca._m14 -= 1.0;
        if (ca._m24 > 0.999) ca._m24 -= 1.0;
        if (ca._m34 > 0.999) ca._m34 -= 1.0;

        if (cb._m14 > 0.999) cb._m14 -= 1.0;
        if (cb._m24 > 0.999) cb._m24 -= 1.0;
        if (cb._m34 > 0.999) cb._m34 -= 1.0;

        return QList<CrystalSymOp>() << ca << cb;
    }

    return QList<CrystalSymOp>();
}

QStringList CrystalSymOp::toStringList() const
{
    QStringList l;

    l.append(QString("%1").arg(_m11, 0, 'f', 8));
    l.append(QString("%1").arg(_m12, 0, 'f', 8));
    l.append(QString("%1").arg(_m13, 0, 'f', 8));
    l.append(QString("%1").arg(_m14, 0, 'f', 8));

    l.append(QString("%1").arg(_m21, 0, 'f', 8));
    l.append(QString("%1").arg(_m22, 0, 'f', 8));
    l.append(QString("%1").arg(_m23, 0, 'f', 8));
    l.append(QString("%1").arg(_m24, 0, 'f', 8));

    l.append(QString("%1").arg(_m31, 0, 'f', 8));
    l.append(QString("%1").arg(_m32, 0, 'f', 8));
    l.append(QString("%1").arg(_m33, 0, 'f', 8));
    l.append(QString("%1").arg(_m34, 0, 'f', 8));

    return l;
}

CrystalAtom CrystalSymOp::transform(const CrystalAtom &atIn)
{
    CrystalAtom atOut = atIn;

    double _ax = _m11 * atIn.x() + _m12 * atIn.y() + _m13 * atIn.z() + _m14;
    double _ay = _m21 * atIn.x() + _m22 * atIn.y() + _m23 * atIn.z() + _m24;
    double _az = _m31 * atIn.x() + _m32 * atIn.y() + _m33 * atIn.z() + _m34;

    atOut.setFcoord_x(_ax);
    atOut.setFcoord_y(_ay);
    atOut.setFcoord_z(_az);

    return atOut;
}
