/***************************************************************************
                          asciihklexport.cpp  -  description
                             -------------------
    begin                : Sat Jan 16, 2016
    copyright            : (C) 2016 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "asciihklexport.h"
#include "bgmnfileio.h"
#include "functions.h"

AsciiHklExport::AsciiHklExport(QObject *parent) :
    GenericExport(parent)
{
    uId = "ASCII_HKL";
}

int AsciiHklExport::save(const QString &file, Scan &scan, const QMap<QString, QVariant> &flags)
{
    QString data = getData(scan, flags.value("fieldSeparator", " ").toString());
    bool b = BgmnFileIO::writeTextFile(file, data);

    if (b) return 1;
    return 0;
}

int AsciiHklExport::save(const QString &file, QVector<Scan> &scanHeap, const QMap<QString, QVariant> &flags)
{
    QString data = getData(scanHeap, flags.value("fieldSeparator", " ").toString());
    bool b = BgmnFileIO::writeTextFile(file, data);

    if (b) return qMax(scanHeap.size() - 4, 1);
    return 0;
}


QString AsciiHklExport::getData(Scan &scan, const QString &sep)
{
    QString data;
    if (!scan.pDataHkl().size()) {
        qDebug() << QString("AsciiHklExport::save(): Scan %1 does not contain HKL data").arg(scan.name());
        return data;
    }

    QTextStream stream(&data);
    stream.setRealNumberNotation(QTextStream::FixedNotation);
    stream.setRealNumberPrecision(6);

    stream << QStringLiteral("\"Phase Name\"") << sep;
    stream << QStringLiteral("\"Diffraction Angle (deg)\"") << sep;
    stream << QStringLiteral("\"Phase No.\"") << sep;
    stream << QStringLiteral("\"HKL\"") << sep;
    stream << QStringLiteral("\"Texture\"\n");

    QString name = QString("\"%1\"").arg(scan.name());

    for (int i = 0; i < scan.pDataHkl().size(); i++) {
        stream << name << sep;
        stream << global::Functions::dToTwoTheta(scan.pDataHkl()[i].position(), 0.1*scan.waveLength()) << sep;
        stream << QStringLiteral("0") << sep;
        stream << QString("\"%1\"").arg(scan.pDataHkl()[i].hkl()) << sep;
        stream << scan.pDataHkl()[i].texture() << "\n";
    }

    return data;
}

QString AsciiHklExport::getData(QVector<Scan> &scanHeap, const QString &sep)
{
    QString data;
    QTextStream stream(&data);
    stream.setRealNumberNotation(QTextStream::FixedNotation);
    stream.setRealNumberPrecision(6);

    stream << QStringLiteral("\"Phase Name\"") << sep;
    stream << QStringLiteral("\"Diffraction Angle (deg)\"") << sep;
    stream << QStringLiteral("\"Phase No.\"") << sep;
    stream << QStringLiteral("\"HKL\"") << sep;
    stream << QStringLiteral("\"Texture\"\n");

    int n = 0;

    for (int j = 0; j < scanHeap.size(); ++j) {
        if (!scanHeap[j].pDataHkl().size()) continue;

        QString name = QString("\"%1\"").arg(scanHeap[j].name());

        for (int i = 0; i < scanHeap[j].pDataHkl().size(); i++) {
            stream << name << sep;
            stream << global::Functions::dToTwoTheta(scanHeap[j].pDataHkl()[i].position(), 0.1*scanHeap[j].waveLength()) << sep;
            stream << n << sep;
            stream << QString("\"%1\"").arg(scanHeap[j].pDataHkl()[i].hkl()) << sep;
            stream << scanHeap[j].pDataHkl()[i].texture() << "\n";
        }
    }

    return data;
}

