/***************************************************************************
                          hkl.cpp  -  description
                             -------------------
    begin                : Wed Mar 04, 2009
    copyright            : (C) 2009 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDataStream>
#include "hkl.h"

Hkl::Hkl()
{
    _uid = QUuid::createUuid();
    _phase = QStringLiteral("");
    _col = Qt::black;
    _pos = 0.0;
    _hkl = "- - -";
    _vpos = 0;
    _intensity = 1.0;
    _texture = 1.0;
    _status = 1;
    _b1 = 0.0;
    _k2 = 0.0;
}

Hkl::Hkl(double pos, const QString &hkl, int vpos, const QString &ph, const QColor &c, double intens, double tex, int stat, double b1, double k2)
{
    _uid = QUuid::createUuid();
    _splitter = "!";
    _pos = pos;
    _hkl = hkl;
    _vpos = vpos;
    _phase = ph;
    _col = c;
    _intensity = intens;
    _texture = tex;
    _status = stat;
    _b1 = b1;
    _k2 = k2;

    // make sure the splitter used for string format does not appear in the phase name
    _phase.replace(_splitter, "_");
}

Hkl::Hkl(const QString &p, const QString &c, const QJsonObject &json)
{
    _uid = QUuid::createUuid();

    _phase     = p;
    _col.setNamedColor(c);

    QJsonValue jv_pos = json["position"];
    _pos       = jv_pos.isUndefined() ? 0.0 : jv_pos.toDouble();

    QJsonValue jv_hkl = json["hkl"];
    _hkl       = jv_hkl.isUndefined() ? QString() : jv_hkl.toString();

    QJsonValue jv_vpos = json["vposition"];
    _vpos      = jv_vpos.isUndefined() ? 0 : json["vposition"].toInt();

    QJsonValue jv_intens = json["intensity"];
    _intensity = jv_intens.isUndefined() ? 0.0 : json["intensity"].toDouble();

    QJsonValue jv_tex = json["texture"];
    _texture   = jv_tex.isUndefined() ? 1.0 : json["texture"].toDouble();

    QJsonValue jv_status = json["status"];
    _status    = jv_status.isUndefined() ? 1 : jv_status.toInt();

    QJsonValue jv_b1 = json["b1"];
    _b1        = jv_b1.isUndefined() ? 0.0 : jv_b1.toDouble();

    QJsonValue jv_k2 = json["k2"];
    _b1        = jv_k2.isUndefined() ? 0.0 : jv_k2.toDouble();
}

QJsonObject Hkl::getJsonData() const
{
    QJsonObject json;
    json["position"]  = _pos;
    json["hkl"]       = _hkl;
    json["vposition"] = _vpos;
    json["intensity"] = _intensity;
    json["texture"]   = _texture;
    json["status"]    = _status;
    json["b1"]        = _b1;
    json["k2"]        = _k2;
    return json;
}
