/***************************************************************************
                          stoeproimport.cpp  -  description
                             -------------------
    begin                : Thu Jun 18, 2013
    copyright            : (C) 2013 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stoeproimport.h"

StoeProImport::StoeProImport(QObject *parent) :
    GenericImport(parent)
{
    extens << "pro";
    descr = QLatin1String("Stoe PRO scan");
}

bool StoeProImport::isSupported(const QByteArray &ba)
{
    QStringList header = QString(ba).split(QRegularExpression("\\r\\n?|\\n"));

    if (header.size() < 2) {
        return false;
    }

    QRegExp rxh("^(?:\\s*\\d+\\.\\d+){2,2}\\s*$");
    QRegExp rxd("^(?:\\s*\\d+\\.\\d*){10,10}\\s*$");

    if (rxh.exactMatch(header.at(0)) && (rxd.exactMatch(header.at(1)))) {
        return true;
    }

    return false;
}

int StoeProImport::load(const QString &file, QVector<Scan> &scanHeap, bool minimal)
{
    Q_UNUSED(minimal);

    qDebug() << QString("StoeProImport::load(): Loading file %1").arg(file);
    QStringList content(BgmnFileIO::readTextFileLines(file));

    if (content.size() < 2) {
        qDebug() << QString("StoeProImport::load(): No valid content found in file %1").arg(file);
        return -1;
    }

    Scan scan(QString(), QColor(), 1);
    scan.setSourceFileName(file);
    QVector<double> vec_a;
    QVector<double> vec_i;

    QStringList line = content.at(0).trimmed().split(QRegExp("\\s+"));

    if (line.size() < 2) {
        qDebug() << QString("StoeProImport::load(): No valid first line found in file %1").arg(file);
        return -1;
    }

    bool ok;
    double startAng = line.at(0).toDouble(&ok);
    double stepSize = line.at(1).toDouble(&ok);
    double ang = startAng;

    QRegExp split("\\.\\s+");

    for (int k = 1; k < content.size(); ++k) {
        line = content.at(k).split(split);

        for (int j = 0; j < line.size(); ++j) {
            vec_a.push_back(ang);
            vec_i.push_back(line.at(j).toDouble(&ok));
            ang += stepSize;
        }
    }

    scan.setDataAng(vec_a);
    scan.setDataInt(vec_i);
    scan.setTypes(Scan::XY | Scan::MEASURED);
    scanHeap.push_back(scan);

    return scanHeap.size();
}
