/***************************************************************************
                          editinstrumentdialog.cpp  -  description
                             -------------------
    begin                : Wed Feb 30 11:00:00 CEST 2013
    copyright            : (C) 2013 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "editinstrumentdialog.h"
#include "editinstrumentparsingerrordialog.h"
#include "ui_editinstrumentdialog.h"
#include "../libXrdIO/bgmnfileio.h"
#include "projectWidget/syntaxHighlighter/bgmnhighlighter.h"
#include "projectWidget/bgmnbackendconfig.h"
#include "../libXrdIO/parser/bgmninstrumentsavparser.h"
#include "../libXrdIO/parser/bgmnsavparser.h"
#include "opticsconfigpageblank.h"
#include "opticsconfigpagegonio.h"
#include "opticsconfigpagefocus.h"
#include "opticsconfigpagepcol.h"
#include "opticsconfigpagescol.h"
#include "opticsconfigpagesslit.h"
#include "opticsconfigpagehslit.h"
#include "opticsconfigpagevslit.h"
#include "opticsconfigpagesample.h"
#include "opticsconfigpagedetector.h"
#include "opticsconfigpagemonochromator.h"
#include "opticsconfigpagetslit.h"
#include "opticsconfigpagerslit.h"
#include "opticsconfigpagedslit.h"
#include "opticsconfigpageairscat.h"

#include <QString>
#include <QtCore>
#include <QtGui>
#include <QtWidgets>
#include <QSplitter>

EditInstrumentDialog::EditInstrumentDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::EditInstrumentDialog)
{
    ui->setupUi(this);
    settings = SettingsManager::getInstance();
    rayTracer = new InstrumentRayTracerMT;
    rayTracer->setProgressBar(ui->progressBar);
    rayTracer->setOutputEditor(ui->textEditOutput);

    for (int i = 0; i < ui->stackedWidgetEditor->count(); ++i) {
        ui->stackedWidgetEditor->widget(i)->layout()->setMargin(0);
    }

    bool dm = settings->isDarkMode();
    scene = new InstrumentScene(dm);
    if (dm) scene->setBackgroundBrush(QBrush(QGuiApplication::palette().color(QPalette::Base)));
    ui->graphicsViewInstrument->setScene(scene);

    ui->tabWidgetParams->widget(0)->layout()->setMargin(0);
    ui->tabWidgetParams->widget(1)->layout()->setMargin(0);

    ui->toolButtonRevertFile->setEnabled(false);

    connect(scene, SIGNAL(itemSelected(QString)), this, SLOT(instrumentItemSelected(QString)));
    connect(ui->textEditFile, SIGNAL(textChanged()), this, SLOT(textDocumentChanged()));
    connect(ui->treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(configItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
    connect(rayTracer, SIGNAL(processComplete()), this, SLOT(calculationComplete()));
    connect(ui->treeWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(toggleItemInstalled(QTreeWidgetItem*,int)));

    QStringList devDirs = settings->value("bgmnProject/deviceDatabase", QStringList()).toStringList();

    QMenu *fileOpenMenu = new QMenu;

    for (int i = 0; i < devDirs.size(); ++i) {
        QAction *act = fileOpenMenu->addAction(devDirs.at(i), this, SLOT(loadFile()));
        act->setData(devDirs.at(i));
    }

    ui->toolButtonLoadFile->setPopupMode(QToolButton::InstantPopup);
    ui->toolButtonLoadFile->setMenu(fileOpenMenu);


    initConfigPages();
    initSettings();

    //setGuiEnabled(false);
    workingDir = QString();
}

EditInstrumentDialog::~EditInstrumentDialog()
{
    if (rayTracer) delete rayTracer;
    if (scene)     delete scene;
    if (ui)        delete ui;
}

void EditInstrumentDialog::showEvent(QShowEvent *e)
{
    e->accept();
    // if (instrSavFile.isEmpty()) loadFile();
}

void EditInstrumentDialog::closeEvent(QCloseEvent *e)
{
    saveSettings();
    e->accept();
}

void EditInstrumentDialog::initConfigPages()
{
    configPageMap.insert("BLANK",         new OpticsConfigPageBlank("BLANK"));
    configPageMap.insert("GONIO",         new OpticsConfigPageGonio("GONIO"));
    configPageMap.insert("FOCUS",         new OpticsConfigPageFocus("FOCUS"));
    configPageMap.insert("PCOLL",         new OpticsConfigPagePcol("PCOLL"));
    configPageMap.insert("SCOLL",         new OpticsConfigPageScol("SCOLL"));
    configPageMap.insert("SSLIT",         new OpticsConfigPageSslit("SSLIT"));
    configPageMap.insert("VSLIT",         new OpticsConfigPageVslit("VSLIT"));
    configPageMap.insert("HSLIT",         new OpticsConfigPageHslit("HSLIT"));
    configPageMap.insert("TSLIT",         new OpticsConfigPageTslit("TSLIT"));
    configPageMap.insert("RSLIT",         new OpticsConfigPageRslit("RSLIT"));
    configPageMap.insert("DSLIT",         new OpticsConfigPageDslit("DSLIT"));
    configPageMap.insert("AIRSCAT",       new OpticsConfigPageAirscat("AIRSCAT"));
    configPageMap.insert("SAMPLE",        new OpticsConfigPageSample("SAMPLE"));
    configPageMap.insert("DETECTOR",      new OpticsConfigPageDetector("DETECTOR"));
    configPageMap.insert("MONOCHROMATOR", new OpticsConfigPageMonochromator("MONOCHROMATOR"));

    QTreeWidgetItem *itmGonio =    new QTreeWidgetItem(QStringList("Goniometer"));
    QTreeWidgetItem *itmPrimBeam = new QTreeWidgetItem(QStringList("Primary Beam"));
    QTreeWidgetItem *itmTube =     new QTreeWidgetItem(QStringList("X-ray tube"));
    QTreeWidgetItem *itmTSlit =    new QTreeWidgetItem(QStringList("Axial slit"));
    QTreeWidgetItem *itmHSlit =    new QTreeWidgetItem(QStringList("Divergence slit"));
    QTreeWidgetItem *itmPColl =    new QTreeWidgetItem(QStringList("Soller slit"));
    QTreeWidgetItem *itmVSlit =    new QTreeWidgetItem(QStringList("Axial beam mask"));
    QTreeWidgetItem *itmRSlit =    new QTreeWidgetItem(QStringList("Pinhole aperture"));

    QTreeWidgetItem *itmSampStage = new QTreeWidgetItem(QStringList("Sample Stage"));
    QTreeWidgetItem *itmSample =   new QTreeWidgetItem(QStringList("Sample"));
    QTreeWidgetItem *itmAirScat =  new QTreeWidgetItem(QStringList("Beam knife"));

    QTreeWidgetItem *itmSecBeam =  new QTreeWidgetItem(QStringList("Secondary Beam"));
    QTreeWidgetItem *itmSSlit =    new QTreeWidgetItem(QStringList("Anti-scatter slit"));
    QTreeWidgetItem *itmSColl =    new QTreeWidgetItem(QStringList("Soller slit"));
    QTreeWidgetItem *itmDSlit =    new QTreeWidgetItem(QStringList("Detector slit"));
    QTreeWidgetItem *itmMono =     new QTreeWidgetItem(QStringList("Monochromator"));
    QTreeWidgetItem *itmDet =      new QTreeWidgetItem(QStringList("Detector"));

    itmPrimBeam->setData(0,  Qt::UserRole, "BLANK");
    itmSecBeam->setData(0,   Qt::UserRole, "BLANK");
    itmSampStage->setData(0, Qt::UserRole, "BLANK");
    itmGonio->setData(0,     Qt::UserRole, "GONIO");
    itmTube->setData(0,      Qt::UserRole, "FOCUS");
    itmPColl->setData(0,     Qt::UserRole, "PCOLL");
    itmSColl->setData(0,     Qt::UserRole, "SCOLL");
    itmSSlit->setData(0,     Qt::UserRole, "SSLIT");
    itmVSlit->setData(0,     Qt::UserRole, "VSLIT");
    itmHSlit->setData(0,     Qt::UserRole, "HSLIT");
    itmTSlit->setData(0,     Qt::UserRole, "TSLIT");
    itmRSlit->setData(0,     Qt::UserRole, "RSLIT");
    itmAirScat->setData(0,   Qt::UserRole, "AIRSCAT");
    itmSample->setData(0,    Qt::UserRole, "SAMPLE");
    itmDSlit->setData(0,     Qt::UserRole, "DSLIT");
    itmDet->setData(0,       Qt::UserRole, "DETECTOR");
    itmMono->setData(0,      Qt::UserRole, "MONOCHROMATOR");

    itmPColl->setCheckState(0, Qt::Checked);
    itmSColl->setCheckState(0, Qt::Checked);
    itmSSlit->setCheckState(0, Qt::Checked);
    itmVSlit->setCheckState(0, Qt::Checked);
    itmHSlit->setCheckState(0, Qt::Checked);
    itmTSlit->setCheckState(0, Qt::Checked);
    itmRSlit->setCheckState(0, Qt::Checked);
    itmAirScat->setCheckState(0, Qt::Checked);
    itmDSlit->setCheckState(0, Qt::Checked);
    itmMono->setCheckState(0, Qt::Checked);

    treeWidgetMap.insert(itmGonio->data(0, Qt::UserRole).toString(),   itmGonio);
    treeWidgetMap.insert(itmTube->data(0, Qt::UserRole).toString(),    itmTube);
    treeWidgetMap.insert(itmPColl->data(0, Qt::UserRole).toString(),   itmPColl);
    treeWidgetMap.insert(itmSColl->data(0, Qt::UserRole).toString(),   itmSColl);
    treeWidgetMap.insert(itmSSlit->data(0, Qt::UserRole).toString(),   itmSSlit);
    treeWidgetMap.insert(itmVSlit->data(0, Qt::UserRole).toString(),   itmVSlit);
    treeWidgetMap.insert(itmHSlit->data(0, Qt::UserRole).toString(),   itmHSlit);
    treeWidgetMap.insert(itmTSlit->data(0, Qt::UserRole).toString(),   itmTSlit);
    treeWidgetMap.insert(itmRSlit->data(0, Qt::UserRole).toString(),   itmRSlit);
    treeWidgetMap.insert(itmAirScat->data(0, Qt::UserRole).toString(), itmAirScat);
    treeWidgetMap.insert(itmSample->data(0, Qt::UserRole).toString(),  itmSample);
    treeWidgetMap.insert(itmDSlit->data(0, Qt::UserRole).toString(),   itmDSlit);
    treeWidgetMap.insert(itmDet->data(0, Qt::UserRole).toString(),     itmDet);
    treeWidgetMap.insert(itmMono->data(0, Qt::UserRole).toString(),    itmMono);

    itmPrimBeam->addChild(itmTube);
    itmPrimBeam->addChild(itmTSlit);
    itmPrimBeam->addChild(itmHSlit);
    itmPrimBeam->addChild(itmPColl);
    itmPrimBeam->addChild(itmVSlit);
    itmPrimBeam->addChild(itmRSlit);

    itmSampStage->addChild(itmSample);
    itmSampStage->addChild(itmAirScat);

    itmSecBeam->addChild(itmSSlit);
    itmSecBeam->addChild(itmSColl);
    itmSecBeam->addChild(itmDSlit);
    itmSecBeam->addChild(itmMono);
    itmSecBeam->addChild(itmDet);

    itmGonio->addChild(itmPrimBeam);
    itmGonio->addChild(itmSampStage);
    itmGonio->addChild(itmSecBeam);

    ui->treeWidget->addTopLevelItem(itmGonio);

    itmGonio->setExpanded(true);
    itmPrimBeam->setExpanded(true);
    itmSampStage->setExpanded(true);
    itmSecBeam->setExpanded(true);

    QMapIterator<QString, AbstractOpticsConfigPage*> it(configPageMap);

    while (it.hasNext()) {
        it.next();
        ui->stackedWidgetParameter->addWidget(it.value());
        configParamHelpText[it.value()->configTag()] = it.value()->helpText();
        connect(it.value(), SIGNAL(updateLayout(QString,QVariant)), scene, SLOT(updateSceneElement(QString,QVariant)));
    }

    if (configPageMap.contains("GONIO")) {
        ui->stackedWidgetParameter->setCurrentWidget(configPageMap.value("GONIO"));
        ui->textEditParameterHelp->setText(configParamHelpText.value("GONIO"));
    }
}

void EditInstrumentDialog::initSettings()
{
    QRect r = settings->value("instrumentDialog/geometry", QRect()).toRect();
    if (!r.isEmpty()) resize(r.width(), r.height());

    ui->splitter->setStretchFactor(0, 10);
    ui->splitter->setStretchFactor(1, 1);

    ui->splitterInstrument->setStretchFactor(0, 1);
    ui->splitterInstrument->setStretchFactor(1, 10);

    ui->splitterParameters->setStretchFactor(0, 10);
    ui->splitterParameters->setStretchFactor(1, 1);

    ui->splitter->restoreState(settings->value("instrumentDialog/splitter", QByteArray()).toByteArray());
    ui->splitterInstrument->restoreState(settings->value("instrumentDialog/splitterInstrument", QByteArray()).toByteArray());
    ui->splitterParameters->restoreState(settings->value("instrumentDialog/splitterParameters", QByteArray()).toByteArray());

    ui->progressBar->setEnabled(false);

    // set up text editors
    BgmnHighlighter *bhlSav = new BgmnHighlighter(settings->getSyntaxHighlightingMode(), this);
    bhlSav->setDocument(ui->textEditFile->document());

    QFont edFont;
    edFont.fromString(settings->value("config/editorFont", font().toString()).toString());
    ui->textEditFile->setFont(edFont);
    ui->textEditOrigFile->setFont(edFont);
    ui->textEditOutput->setFont(edFont);

    ui->toolButtonToggleControlFileEditor->setChecked(true);
    ui->stackedWidgetEditor->setCurrentIndex(1);
}

void EditInstrumentDialog::saveSettings()
{
    QMapIterator<QString, AbstractOpticsConfigPage *> iter(configPageMap);

    while (iter.hasNext()) {
        iter.next();
        iter.value()->saveSettings();
    }

    settings->setValue("instrumentDialog/geometry", geometry());
    settings->setValue("instrumentDialog/splitter", ui->splitter->saveState());
    settings->setValue("instrumentDialog/splitterInstrument", ui->splitterInstrument->saveState());
    settings->setValue("instrumentDialog/splitterParameters", ui->splitterParameters->saveState());
}

void EditInstrumentDialog::loadFile()
{
    QString dir;

    QAction *action = qobject_cast<QAction *>(sender());
    if (action) dir = action->data().toString();

    QString f = QFileDialog::getOpenFileName(this,
                                             tr("Load instrument configuration"),
                                             dir,
                                             tr("Instrument configuration (*.sav *.SAV)"));
    if (!f.isEmpty()) {
        resetDialog();
        setGuiEnabled(true);
        openInstrumentSavFile(f);
    }
}

void EditInstrumentDialog::textDocumentChanged()
{
}

void EditInstrumentDialog::setUiStateRunning(RunState r)
{
    bool b = r == RunState::running;
    ui->progressBar->setEnabled(b);
    ui->stackedWidgetEditor->setEnabled(!b);
    ui->topButtonBox->setEnabled(!b);
    ui->toolButtonLoadFile->setEnabled(!b);
    ui->pushButtonClose->setEnabled(!b);

    if (b) {
        ui->toolButtonRun->setIcon(QIcon::fromTheme("profex-run-abort"));
        ui->textEditOutput->clear();
        ui->progressBar->reset();
        ui->progressBar->setValue(0);
        ui->textEditOutput->appendPlainText(QString(tr("Computation started. Please be patient...")));
        qDebug() << QString("EditInstrumentDialog::setUiStateRunning(): Profile calculation started");
    } else {
        ui->toolButtonRun->setIcon(QIcon::fromTheme("profex-run"));
        ui->progressBar->reset();

        if (r == RunState::complete) {
            ui->textEditOutput->appendPlainText(QString(tr("Calculation completed")));
            qDebug() << QString("EditInstrumentDialog::setUiStateRunning(): Profile calculation completed");
        } else {
            ui->textEditOutput->appendPlainText(QString(tr("Calculation aborted")));
            qDebug() << QString("EditInstrumentDialog::setUiStateRunning(): Profile calculation was aborted");
        }
    }
}

void EditInstrumentDialog::toggleItemInstalled(QTreeWidgetItem *itm, int)
{
    QString tag = itm->data(0, Qt::UserRole).toString();
    scene->itemActiveStatusChanged(tag, itm->checkState(0) == Qt::Checked);

    if (configPageMap.contains(tag)) {
        configPageMap.value(tag)->setInstalled(itm->checkState(0) == Qt::Checked);
    }
}

void EditInstrumentDialog::toggleOrigFileEditor(bool b)
{
    if (!b) return;
    raiseOriginalFileEditor();
}

void EditInstrumentDialog::toggleControlFileEditor(bool b)
{
    if (!b) return;

    if (ui->stackedWidgetEditor->currentIndex() == 0) {
        // changing from graphical editor to text editor
        ui->textEditFile->setPlainText(updateControlFile());
    }

    raiseControlFileEditor();
}

void EditInstrumentDialog::toggleGraphEditor(bool b)
{
    if (!b) return;
    bool doToggle = true;

    if (ui->stackedWidgetEditor->currentIndex() == 1) {
        // changing from the control text editor to the graphical editor
        QMap<QString, QString> parsingErrors = parseControlFile(false);

        if (!parsingErrors.isEmpty()) {
            EditInstrumentParsingErrorDialog *errDlg = new EditInstrumentParsingErrorDialog(1, this);
            errDlg->setErrorMap(parsingErrors);

            if (errDlg->exec() == QDialog::Rejected) {
                doToggle = false;
            }

            delete errDlg;
        }
    }

    if (doToggle) raiseGraphicalEditor();
    else          raiseControlFileEditor();
}

void EditInstrumentDialog::fileSave()
{
    save(instrSavFile);
}

void EditInstrumentDialog::openInstrumentSavFile(const QString &s)
{
    origInstrSavFileContent = BgmnFileIO::readTextFile(s);
    ui->textEditFile->setPlainText(origInstrSavFileContent);
    ui->textEditOrigFile->setPlainText(origInstrSavFileContent);
    instrSavFile = s;

    QFileInfo isv(instrSavFile);
    controlSavTemplate = QString("%1/%2.tpl").arg(isv.absolutePath()).arg(isv.completeBaseName());

    ui->textEditFile->moveCursor(QTextCursor::Start);
    setWindowTitle(QString(tr("Instrument Configuration - %1")).arg(instrSavFile));

    QMap<QString, QString> parsingErrors = parseControlFile(true);
    bool raiseGraph = true;

    if (!parsingErrors.isEmpty()) {
        EditInstrumentParsingErrorDialog *errDlg = new EditInstrumentParsingErrorDialog(0, this);
        errDlg->setErrorMap(parsingErrors);
        errDlg->exec();
        raiseGraph = false;
        delete errDlg;
    }

    if (raiseGraph) raiseGraphicalEditor();
    else            raiseControlFileEditor();
    setGuiEnabled(true);
}

void EditInstrumentDialog::fileReset()
{
    if (QMessageBox::question(this,
                              tr("Reset control file"),
                              tr("Reset the control file to the original version?\n"
                                 "All changes will be discarded."))
            == QMessageBox::No)
        return;

    ui->textEditFile->setPlainText(origInstrSavFileContent);
    raiseControlFileEditor();
}

void EditInstrumentDialog::runCalculation()
{
    if (rayTracer->isRunning()) {
        rayTracer->abortProcess();
        setUiStateRunning(RunState::aborted);
        return;
    }

    if (instrSavFile.isEmpty()) {
        fileSaveAs();
        if (instrSavFile.isEmpty()) return;
    }

    setUiStateRunning(RunState::running);

    if (ui->stackedWidgetEditor->currentIndex() == 0) {
        ui->textEditFile->setPlainText(updateControlFile());
    }

    if (!checkOutputNames()) fixOutputNames();

    save(instrSavFile);
    saveTemplate();

    rayTracer->setConfigFile(ui->textEditFile->toPlainText(), instrSavFile);
    rayTracer->runProcess();
}

void EditInstrumentDialog::calculationComplete()
{
    setUiStateRunning(RunState::complete);
}

bool EditInstrumentDialog::checkOutputNames()
{
    BgmnInstrumentSavParser isParser(ui->textEditFile->toPlainText(), instrSavFile);
    return isParser.checkOutputNames(instrSavFile);
}

void EditInstrumentDialog::fixOutputNames()
{
    BgmnInstrumentSavParser isParser(ui->textEditFile->toPlainText(), instrSavFile);
    isParser.fixOutputNames(instrSavFile);
    ui->textEditFile->setPlainText(isParser.getText());
}

void EditInstrumentDialog::fileSaveAs()
{
    BgmnBackendConfig bkgConfig;
    QString userRepoDir = bkgConfig.getUserDeviceRepo();

    QString fout = QFileDialog::getSaveFileName(this,
                                                tr("Save File as"),
                                                userRepoDir,
                                                tr("Control File (*.SAV *.sav)"));

    if (fout.isEmpty()) return;

    instrSavFile = fout;

    QFileInfo isv(instrSavFile);
    controlSavTemplate = QString("%1/%2.tpl").arg(isv.absolutePath()).arg(isv.completeBaseName());

    setWindowTitle(QString(tr("Instrument Configuration - %1")).arg(instrSavFile));
    save(instrSavFile);
}

void EditInstrumentDialog::save(const QString &s)
{
    qDebug() << QString("EditInstrumentDialog::save(): Saving instrument file to %1").arg(s);
    BgmnFileIO::writeTextFile(s, ui->textEditFile->toPlainText());
}

void EditInstrumentDialog::saveTemplate()
{
    QString tplStr;
    double monAng;

    if (configPageMap.contains("MONOCHROMATOR")) {
        OpticsConfigPageMonochromator *monPage = dynamic_cast<OpticsConfigPageMonochromator*>(configPageMap.value("MONOCHROMATOR"));
        if (!monPage) return;
        monAng = monPage->getMonAngle();
    }

    if (configPageMap.contains("FOCUS")) {
        OpticsConfigPageFocus *focPage = dynamic_cast<OpticsConfigPageFocus*>(configPageMap.value("FOCUS"));
        if (!focPage) return;
        tplStr = focPage->getTemplate(monAng);
    }

    QFileInfo fi(instrSavFile);
    controlSavTemplate = QString("%1/%2.tpl").arg(fi.absolutePath()).arg(fi.completeBaseName());

    qDebug() << QString("EditInstrumentDialog::saveTemplate(): Writing template to file %1").arg(controlSavTemplate);
    BgmnFileIO::writeTextFile(controlSavTemplate, tplStr);
}

/*
 * the control file or the template control file is parsed to determine the wavelength / lambda file
 */
void EditInstrumentDialog::parseTemplate(QMap<QString, QString> &params)
{
    // if the "current" instrument configuration is edited, the file "controlSavFile" instead
    // of the template should be parsed. OTOH, if an instrument configuration from the repo is
    // edited, "controlSavFile" is explicitly set to QString() and we should parse "controlSavTemplate"

    QString cFile = controlSavFile.isEmpty() ? controlSavTemplate : controlSavFile;

    if (!QFile::exists(cFile)) {
        qDebug() << QString("EditInstrumentDialog::parseTemplate: Template doesn't exist %1").arg(cFile);
        return;
    }

    qDebug() << QString("EditInstrumentDialog::parseTemplate: Parsing template %1").arg(cFile);

    BgmnSavParser sparser(cFile);
    double syn = sparser.getSynchrotron();
    QString lam = sparser.lamFile();

    if (syn > 0.0) {
        params.insert("SYNCHROTRON", QString("%1").arg(syn, 0, 'f', 8));
        qDebug() << QString("   Found SYNCHROTRON=%1").arg(params.value("SYNCHROTRON"));
    } else if (!lam.isEmpty()) {
        params.insert("LAMBDA", lam);
        qDebug() << QString("   Found LAMBDA=%1").arg(params.value("LAMBDA"));
    }
}

/*
 * updates the graphicsscene parameters based on the control file
 */
QMap<QString, QString> EditInstrumentDialog::parseControlFile(bool tpl)
{
    BgmnInstrumentSavParser isParser(ui->textEditFile->toPlainText(), instrSavFile);
    header = isParser.getHeader();

    QMap<QString, QString> params = isParser.getParameters();
    if (tpl) parseTemplate(params);

    QFileInfo fi(instrSavFile);

    params.insert("VERZERR", QString("%1.ger").arg(fi.completeBaseName()));
    params.insert("GEQ",     QString("%1.geq").arg(fi.completeBaseName()));

    QMap<QString, QString> paramErrors;
    QMapIterator<QString, AbstractOpticsConfigPage*> itCPage(configPageMap);

    while (itCPage.hasNext()) {
        itCPage.next();
#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
        paramErrors.insert(itCPage.value()->setParameters(params));
#else
        paramErrors.unite(itCPage.value()->setParameters(params));
#endif
    }

    QMap<QString, bool> checkStateMap;

    // GONIO permanently installed
    // FOCUS permanently installed
    // SAMPLE permanently installed
    // DETECTOR permanently installed
    checkStateMap["PCOLL"] = params.contains("PColl");
    checkStateMap["SCOLL"] = params.contains("SColl");
    checkStateMap["SSLIT"] = params.contains("SSlitR");
    checkStateMap["VSLIT"] = params.contains("VSlitH");
    checkStateMap["HSLIT"] = params.contains("HSlitR");
    checkStateMap["TSLIT"] = params.contains("TSlitR");
    checkStateMap["RSLIT"] = params.contains("RoundSlitR");
    checkStateMap["AIRSCAT"] = params.contains("AirScat");
    checkStateMap["DSLIT"] = params.contains("DetW");
    checkStateMap["MONOCHROMATOR"] = params.contains("MonR");

    bool oldState = ui->treeWidget->blockSignals(true);

    QMapIterator<QString, bool> itCheckState(checkStateMap);

    while (itCheckState.hasNext()) {
        itCheckState.next();
        treeWidgetMap.value(itCheckState.key())->setCheckState(0, itCheckState.value() ? Qt::Checked : Qt::Unchecked);

        if (configPageMap.contains(itCheckState.key())) {
            configPageMap.value(itCheckState.key())->setInstalled(itCheckState.value());
        }

        scene->itemActiveStatusChanged(itCheckState.key(), itCheckState.value());
    }

    scene->updateSceneElement("SAMPLE", params.value("GEOMETRY"));
    ui->treeWidget->blockSignals(oldState);
    return paramErrors;
}

/*
 * updates the control file based on the graphicsScene
 */
QString EditInstrumentDialog::updateControlFile()
{
    QMap<QString, QString> params;
    QMapIterator<QString, AbstractOpticsConfigPage*> itCPage(configPageMap);

    while (itCPage.hasNext()) {
        itCPage.next();

        QMap<QString, QString> currentParams = itCPage.value()->getParameters();
        QMapIterator<QString, QString> itCParams(currentParams);

        while (itCParams.hasNext()) {
            itCParams.next();
            params.insert(itCParams.key(), itCParams.value());
        }
    }

    BgmnInstrumentSavParser isParser(params, header, instrSavFile);
    return isParser.getText();
}

void EditInstrumentDialog::instrumentItemSelected(QString mod)
{
    QString tag = configPageMap.contains(mod) ? mod : "GONIO";

    bool oldState = ui->treeWidget->blockSignals(true);

    scene->setItemSelected(tag);
    ui->treeWidget->setCurrentItem(treeWidgetMap.value(tag));

    if (configPageMap.contains(tag)) {
        ui->stackedWidgetParameter->setCurrentWidget(configPageMap.value(tag));
    }

    ui->textEditParameterHelp->setText(configParamHelpText.value(tag));

    ui->treeWidget->blockSignals(oldState);
}

void EditInstrumentDialog::configItemChanged(QTreeWidgetItem *it, QTreeWidgetItem *)
{
    instrumentItemSelected(it->data(0, Qt::UserRole).toString());
}

void EditInstrumentDialog::resetDialog(bool disable)
{
    instrSavFile = QString();
    origInstrSavFileContent = QString();
    controlSavFile = QString();
    controlSavTemplate = QString();
    header = QString();

    if (disable) {
        setGuiEnabled(false);
    }
}

void EditInstrumentDialog::setGuiEnabled(bool b)
{
    ui->treeWidget->setEnabled(b);
    ui->tabWidgetParams->setEnabled(b);
    ui->textEditOutput->setEnabled(b);

    ui->stackedWidgetEditor->setEnabled(b);
    ui->topButtonBox->setEnabled(b);
    ui->toolButtonLoadFile->setEnabled(true);
}

void EditInstrumentDialog::blockToggleButtonSignals(bool b)
{
    ui->toolButtonToggleGraphicalEditor->blockSignals(b);
    ui->toolButtonToggleControlFileEditor->blockSignals(b);
    ui->toolButtonToggleOrigFileEditor->blockSignals(b);
    ui->toolButtonRevertFile->blockSignals(b);
    ui->stackedWidgetEditor->blockSignals(b);
}

void EditInstrumentDialog::raiseGraphicalEditor()
{
    blockToggleButtonSignals(true);
    ui->toolButtonToggleGraphicalEditor->setChecked(true);
    ui->toolButtonToggleControlFileEditor->setChecked(false);
    ui->toolButtonToggleOrigFileEditor->setChecked(false);
    ui->toolButtonRevertFile->setEnabled(false);

    ui->stackedWidgetEditor->setCurrentIndex(0);
    blockToggleButtonSignals(false);
}

void EditInstrumentDialog::raiseControlFileEditor()
{
    blockToggleButtonSignals(true);
    ui->toolButtonToggleGraphicalEditor->setChecked(false);
    ui->toolButtonToggleControlFileEditor->setChecked(true);
    ui->toolButtonToggleOrigFileEditor->setChecked(false);
    ui->toolButtonRevertFile->setEnabled(false);

    ui->stackedWidgetEditor->setCurrentIndex(1);
    blockToggleButtonSignals(false);
}

void EditInstrumentDialog::raiseOriginalFileEditor()
{
    blockToggleButtonSignals(true);
    ui->toolButtonToggleGraphicalEditor->setChecked(false);
    ui->toolButtonToggleControlFileEditor->setChecked(false);
    ui->toolButtonToggleOrigFileEditor->setChecked(true);
    ui->toolButtonRevertFile->setEnabled(true);

    ui->stackedWidgetEditor->setCurrentIndex(2);
    blockToggleButtonSignals(false);
}

