/***************************************************************************
                          editinstrumentdialog.h  -  description
                             -------------------
    begin                : Wed Feb 30 11:00:00 CEST 2013
    copyright            : (C) 2013 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EDITINSTRUMENTDIALOG_H
#define EDITINSTRUMENTDIALOG_H

#include "projectWidget/processhandler.h"
#include "instrumentscene.h"
#include <QString>
#include <QMap>
#include <QList>
#include <QStringList>
#include <QDialog>
#include <QTreeWidgetItem>
#include "../libXrdIO/settingsmanager.h"
#include "abstractopticsconfigpage.h"
#include "instrumentraytracermt.h"

namespace Ui {
class EditInstrumentDialog;
}

enum RunState {idle, running, complete, aborted};

class EditInstrumentDialog : public QDialog
{
    Q_OBJECT
    
public:
    explicit EditInstrumentDialog(QWidget *parent = 0);
    ~EditInstrumentDialog();

    void resetDialog(bool disable = false);
    void openInstrumentSavFile(const QString &);
    inline void setControlSavFile(const QString &s) {controlSavFile = s;}
    inline void setWorkingDir(const QString &s)     {workingDir = s;}

    QString getWorkingDir() {return workingDir;}
    
private:
    Ui::EditInstrumentDialog *ui;

    SettingsManager *settings;
    InstrumentScene *scene;
    InstrumentRayTracerMT *rayTracer;
    QString workingDir;
    QString instrSavFile;
    QString origInstrSavFileContent;
    QString controlSavFile;
    QString controlSavTemplate;
    QString header;
    bool useWine;
    bool abort;
    QMap<QString, AbstractOpticsConfigPage *> configPageMap;
    QMap<QString, QTreeWidgetItem *> treeWidgetMap;
    QMap<QString, QString> configParamHelpText;

    void initConfigPages();
    void initSettings();
    void saveSettings();
    void showEvent(QShowEvent *);
    void closeEvent(QCloseEvent *);
    void save(const QString &);
    void saveTemplate();
    bool checkOutputNames();
    void fixOutputNames();
    void setUiStateRunning(RunState);
    void parseTemplate(QMap<QString, QString> &);
    QMap<QString, QString> parseControlFile(bool tpl);
    QString updateControlFile();
    void setGuiEnabled(bool);
    void blockToggleButtonSignals(bool);

    void raiseGraphicalEditor();
    void raiseControlFileEditor();
    void raiseOriginalFileEditor();

private slots:
    void toggleItemInstalled(QTreeWidgetItem*, int);
    void toggleGraphEditor(bool);
    void toggleControlFileEditor(bool);
    void toggleOrigFileEditor(bool);

    void runCalculation();
    void calculationComplete();
    void fileSave();
    void fileSaveAs();
    void fileReset();
    void instrumentItemSelected(QString);
    void textDocumentChanged();
    void loadFile();
    void configItemChanged(QTreeWidgetItem*, QTreeWidgetItem*);
};

#endif // EDITINSTRUMENTDIALOG_H
