/***************************************************************************
                          instrumentraytracer.h  -  description
                             -------------------
    begin                : Sat Jul 25 11:00:00 CEST 2020
    copyright            : (C) 2020 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INSTRUMENTRAYTRACER_H
#define INSTRUMENTRAYTRACER_H

#include "../libXrdIO/settingsmanager.h"
#include "projectWidget/processhandler.h"
#include <QObject>
#include <QProgressBar>
#include <QPlainTextEdit>

struct TtRange{
    int steps;
    double ttStart;
    double ttEnd;
    double wmin;
    double wmax;
};

class InstrumentRayTracer : public QObject
{
    Q_OBJECT

public:
    explicit InstrumentRayTracer(QObject *parent = nullptr);
    ~InstrumentRayTracer();

    void setProgressBar(QProgressBar *);
    void setOutputEditor(QPlainTextEdit *);
    void setConfigFile(const QString &cont, const QString &fn);

public slots:
    void runProcess();
    void processAborted();

private:
    SettingsManager *settings;
    QProgressBar *progressBar;
    QPlainTextEdit *outputEditor;

    ProcessHandler *geomet;
    ProcessHandler *makegeq;

    QString geometExec;
    QString makeGeqExec;

    QString savFileContent;
    QString savFileName;

    TtRange progressRanges;
    bool abort;

    void initProcesses();

    TtRange getRanges();
    double getCurrentAngle(const QString &);
    void applyDistortionCorrection();
    QString getGerName();
    QString getSavVariable(const QString &, const QString &);
    QString bgmnMathToJS(const QString &);

private slots:
    void pollOutput();
    void geometComplete();
    void makegeqComplete();

signals:
    void processComplete();
};

#endif // INSTRUMENTRAYTRACER_H
