/***************************************************************************
                          electrondensitymapdialog.cpp  -  description
                             -------------------
    begin                : Fri May 31 15:00:00 CEST 2019
    copyright            : (C) 2019 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ELECTRONDENSITYMAPDIALOG_H
#define ELECTRONDENSITYMAPDIALOG_H

#include <QtGui>
#include <QDialog>
#include <QProgressDialog>
#include "threadrenderemap.h"
#include "emapwidget2d.h"
#include "emapscale2d.h"
#include "../libXrdIO/colorMaps/lutstructs.h"
#include "emapstructs.h"
#include "../libXrdIO/settingsmanager.h"

namespace Ui {
class ElectronDensityMapDialog;
}

class ElectronDensityMapDialog : public QDialog
{
    Q_OBJECT

public:
    explicit ElectronDensityMapDialog(QWidget *parent = nullptr);
    ~ElectronDensityMapDialog();

    inline void setWorkingDir(const QString &s) {workingDir = s;}
    void setFileName(const QString &);

private:
    Ui::ElectronDensityMapDialog *ui;
    SettingsManager *settings;

    QString workingDir;
    QFileInfo hklFileName;
    QFileInfo cellFileName;
    QMap<QString, colorMaps::Lut> lutList;
    float dmax, dmin;
    QVector< QVector< QVector<float> > > vmap;
    QList<Atom> atoms;
    int activeThreads;
    int nextZLevel;
    int nCpus;
    int bufferMode;
    int levelsX;
    int levelsY;
    int levelsZ;
    QTime startTime;
    bool abort;
    QProgressDialog *pdlg;
    UnitCell cell;
    QMatrix4x4 mf2c; // fractional -> cartesian
    projection_t projection;

    void initSettings();
    void saveSettings();
    void initGuiElements();
    void initConnections();

    QMatrix4x4 createF2CMatrix(const UnitCell &, projection_t);

    void dragEnterEvent(QDragEnterEvent *);
    void dropEvent(QDropEvent *);
    void mousePressEvent(QMouseEvent *);
    void startFourierSynth();

private slots:
    void fileLoad();
    void fileSaveAs();
    void fileSaveAllAs();
    void mergeImagesAverage();
    void mergeImagesMax();

    void calculateMap();
    void currentLevelChanged(int);
    void fourierSyntThreadCompleted();
    void renderMapImage();

    void coordinates(float,float,float);
    void setGuiState(bool);

    void newDmax(float);
    void newDmin(float);
    void dmaxChanged(double);
    void abortCalculation();
    void setProjection(int);
    void lutChanged(QString);
    void settingsDialog();
};

#endif // ELECTRONDENSITYMAPDIALOG_H
