/***************************************************************************
                          emapscale2d.cpp  -  description
                             -------------------
    begin                : Wed Sep 24 20:35:00 CEST 2014
    copyright            : (C) 2014 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "emapscale2d.h"
#include "math.h"
#include "imageeffects.h"
#include <QPainter>

EMapScale2D::EMapScale2D(QWidget *parent) :
    QWidget(parent)
{
    setMinimumWidth(fontMetrics().width("aaa -mm.mm"));
    minE = -1.0;
    maxE = 1.0;
}

void EMapScale2D::setRange(float min, float max)
{
    maxE = qMax(fabs(max), fabs(min));
    minE = -maxE;
}

void EMapScale2D::setLut(const colorMaps::Lut &l)
{
    lut = l;
    updateGradient();
    update();
}

void EMapScale2D::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    if (gradient.isNull()) {
        painter.fillRect(this->rect(), Qt::white);
        return;
    }

    int t = fontMetrics().width("aaa ");

    painter.drawImage(0, 0, gradient);
    painter.drawText(t, fontMetrics().ascent(), QString(" %1").arg(maxE, 6, 'f', 2));
    painter.drawText(t, (height() + fontMetrics().ascent())/2, QString("   0.00"));
    painter.drawText(t, height(), QString(" %1").arg(minE, 6, 'f', 2));
}

void EMapScale2D::resizeEvent(QResizeEvent *)
{
    updateGradient();
}

void EMapScale2D::updateGradient()
{
    if ((width() <= 0) || (height() <= 0)) return;

    float f = float(height()) / float(lut.data.size());
    int w = fontMetrics().width("aaa");
    QImage img(w, height(), QImage::Format_ARGB32);
    QPainter pxpainter(&img);

    for (int i = 0; i < lut.data.size(); ++i) {
        pxpainter.fillRect(QRectF(0.0, float(height()) - (i * f), float(w), -f), QBrush(QColor(lut.data.at(i))));
    }

    pxpainter.end();

    if (lut.contour) {
        if (lut.type == 1) ImageEffects::contourLinesBlackOnColor(&img, lut.contour);
        if (lut.type == 2) ImageEffects::contourLinesBlackOnWhite(&img, lut.contour);
        if (lut.type == 3) ImageEffects::contourLinesColor(&img, lut.contour);
    }

    gradient = img;
}
