/***************************************************************************
                          emapsettingsdialog.h  -  description
                             -------------------
    begin                : Wed Sep 24 20:35:00 CEST 2014
    copyright            : (C) 2014 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "emapsettingsdialog.h"
#include "ui_emapsettingsdialog.h"
#include "../libXrdIO/structs.h"
#include <QDoubleSpinBox>

EmapSettingsDialog::EmapSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::EmapSettingsDialog)
{
    ui->setupUi(this);
    settings = SettingsManager::getInstance();

    initGui();
    initSettings();

}

EmapSettingsDialog::~EmapSettingsDialog()
{
    delete ui;
}

void EmapSettingsDialog::accept()
{
    saveSettings();
    QDialog::accept();
}

void EmapSettingsDialog::initGui()
{
    ui->doubleSpinBoxResolution->setMinimum(0.01);
    ui->doubleSpinBoxResolution->setMaximum(1.0);
    ui->doubleSpinBoxResolution->setSingleStep(0.02);
    ui->doubleSpinBoxResolution->setSuffix(QString(" %1").arg(global::angstrom));

    ui->comboBoxInterpolation->addItem("None");
    ui->comboBoxInterpolation->addItem("Medium");
    ui->comboBoxInterpolation->addItem("High");

    ui->spinBoxImageHeight->setMinimum(256);
    ui->spinBoxImageHeight->setMaximum(10240);

    ui->comboBoxSuperSampling->addItem("off", 1);
    ui->comboBoxSuperSampling->addItem("2x", 2);
    ui->comboBoxSuperSampling->addItem("4x", 4);
}

void EmapSettingsDialog::initSettings()
{
    ui->doubleSpinBoxResolution->setValue(settings->value("emapDialog/levels", 0.25).toDouble());
    ui->comboBoxInterpolation->setCurrentIndex(settings->value("emapDialog/interpolation", 2).toInt());
    ui->spinBoxImageHeight->setValue(settings->value("emapDialog/imageHeight", 768).toInt());

    int ssampl = settings->value("emapDialog/superSampling", 1).toInt();
    if (ssampl == 1) ui->comboBoxSuperSampling->setCurrentIndex(0);
    if (ssampl == 2) ui->comboBoxSuperSampling->setCurrentIndex(1);
    if (ssampl == 4) ui->comboBoxSuperSampling->setCurrentIndex(2);
}

void EmapSettingsDialog::saveSettings()
{
    settings->setValue("emapDialog/levels", ui->doubleSpinBoxResolution->value());
    settings->setValue("emapDialog/interpolation", ui->comboBoxInterpolation->currentIndex());
    settings->setValue("emapDialog/imageHeight", ui->spinBoxImageHeight->value());
    settings->setValue("emapDialog/superSampling", ui->comboBoxSuperSampling->currentData(Qt::UserRole).toInt());
}
