/***************************************************************************
                          prefpagebgmndirectories.cpp  -  description
                             -------------------
    begin                : Sun Aug 06 09:40:00 CEST 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefpagebgmndirectories.h"
#include "ui_prefpagebgmndirectories.h"

#include <QFileInfo>
#include <QFileDialog>

PrefPageBgmnDirectories::PrefPageBgmnDirectories(QWidget *parent) :
    PrefPageTemplate(parent),
    ui(new Ui::PrefPageBgmnDirectories)
{
    ui->setupUi(this);
    initUi();
    initSettings();
}

PrefPageBgmnDirectories::~PrefPageBgmnDirectories()
{
    delete ui;
}

void PrefPageBgmnDirectories::initUi()
{
    connect(ui->toolButtonBgmnStrFilesAdd,    SIGNAL(clicked(bool)), this, SLOT(addBgmnStrDir()));
    connect(ui->toolButtonBgmnStrFilesRemove, SIGNAL(clicked(bool)), this, SLOT(removeBgmnStrDir()));
    connect(ui->toolButtonBgmnDevFilesAdd,    SIGNAL(clicked(bool)), this, SLOT(addBgmnDevDir()));
    connect(ui->toolButtonBgmnDevFilesRemove, SIGNAL(clicked(bool)), this, SLOT(removeBgmnDevDir()));
    connect(ui->toolButtonBgmnPresetsAdd,     SIGNAL(clicked(bool)), this, SLOT(addBgmnPresetDir()));
    connect(ui->toolButtonBgmnPresetsRemove,  SIGNAL(clicked(bool)), this, SLOT(removeBgmnPresetDir()));
}

void PrefPageBgmnDirectories::initSettings()
{
    ui->listWidgetBgmnStrFiles->clear();
    ui->listWidgetBgmnDevFiles->clear();
    ui->listWidgetBgmnPresets->clear();

    addBgmnStrDirs(settings->value("bgmnProject/structureDatabase", QStringList()).toStringList());
    addBgmnDevDirs(settings->value("bgmnProject/deviceDatabase", QStringList()).toStringList());
    addBgmnPresetDirs(settings->value("bgmnProject/presetDirectory", QStringList()).toStringList());
}

void PrefPageBgmnDirectories::saveSettings()
{
    QStringList strFileDirs;
    QStringList devFileDirs;
    QStringList presFileDirs;

    for (int i = 0; i < ui->listWidgetBgmnStrFiles->count(); ++i) {
        strFileDirs.append(ui->listWidgetBgmnStrFiles->item(i)->text());
    }

    for (int i = 0; i < ui->listWidgetBgmnDevFiles->count(); ++i) {
        devFileDirs.append(ui->listWidgetBgmnDevFiles->item(i)->text());
    }

    for (int i = 0; i < ui->listWidgetBgmnPresets->count(); ++i) {
        presFileDirs.append(ui->listWidgetBgmnPresets->item(i)->text());
    }

    settings->setValue("bgmnProject/structureDatabase", strFileDirs);
    settings->setValue("bgmnProject/deviceDatabase", devFileDirs);
    settings->setValue("bgmnProject/presetDirectory", presFileDirs);
}

void PrefPageBgmnDirectories::addBgmnStrDir()
{
    QString s = QDir::homePath();

    if (ui->listWidgetBgmnStrFiles->currentItem()) {
        s = ui->listWidgetBgmnStrFiles->currentItem()->text();
    } else {
        if (ui->listWidgetBgmnStrFiles->count()) {
            s = ui->listWidgetBgmnStrFiles->item(0)->text();
        }
    }

    s = QFileDialog::getExistingDirectory(this, tr("Select structure file directory"), s);

    if (!s.isEmpty()) {
        addBgmnStrDirs(QStringList(s));
    }
}

void PrefPageBgmnDirectories::addBgmnDevDir()
{
    QString s = QDir::homePath();

    if (ui->listWidgetBgmnDevFiles->currentItem()) {
        s = ui->listWidgetBgmnDevFiles->currentItem()->text();
    } else {
        if (ui->listWidgetBgmnDevFiles->count()) {
            s = ui->listWidgetBgmnDevFiles->item(0)->text();
        }
    }

    s = QFileDialog::getExistingDirectory(this, tr("Select device file directory"), s);

    if (!s.isEmpty()) {
        addBgmnDevDirs(QStringList(s));
    }
}

void PrefPageBgmnDirectories::addBgmnPresetDir()
{
    QString s = QDir::homePath();

    if (ui->listWidgetBgmnPresets->currentItem()) {
        s = ui->listWidgetBgmnPresets->currentItem()->text();
    } else {
        if (ui->listWidgetBgmnPresets->count()) {
            s = ui->listWidgetBgmnPresets->item(0)->text();
        }
    }

    s = QFileDialog::getExistingDirectory(this, tr("Select preset directory"), s);

    if (!s.isEmpty()) {
        addBgmnPresetDirs(QStringList(s));
    }
}

void PrefPageBgmnDirectories::removeBgmnStrDir()
{
    QListWidgetItem *it = ui->listWidgetBgmnStrFiles->takeItem(ui->listWidgetBgmnStrFiles->currentRow());
    if (it) delete it;
}

void PrefPageBgmnDirectories::removeBgmnDevDir()
{
    QListWidgetItem *it = ui->listWidgetBgmnDevFiles->takeItem(ui->listWidgetBgmnDevFiles->currentRow());
    if (it) delete it;
}

void PrefPageBgmnDirectories::removeBgmnPresetDir()
{
    QListWidgetItem *it = ui->listWidgetBgmnPresets->takeItem(ui->listWidgetBgmnPresets->currentRow());
    if (it) delete it;
}

void PrefPageBgmnDirectories::addBgmnStrDirs(const QStringList &l)
{
    for (int i = 0; i < l.size(); ++i) {
        QListWidgetItem *it = new QListWidgetItem(l.at(i), ui->listWidgetBgmnStrFiles, 0);
        it->setFlags(it->flags () | Qt::ItemIsEditable);
        ui->listWidgetBgmnStrFiles->addItem(it);

    }
}

void PrefPageBgmnDirectories::addBgmnDevDirs(const QStringList &l)
{
    for (int i = 0; i < l.size(); ++i) {
        QListWidgetItem *it = new QListWidgetItem(l.at(i), ui->listWidgetBgmnDevFiles, 0);
        it->setFlags(it->flags () | Qt::ItemIsEditable);
        ui->listWidgetBgmnDevFiles->addItem(it);

    }
}

void PrefPageBgmnDirectories::addBgmnPresetDirs(const QStringList &l)
{
    for (int i = 0; i < l.size(); ++i) {
        QListWidgetItem *it = new QListWidgetItem(l.at(i), ui->listWidgetBgmnPresets, 0);
        it->setFlags(it->flags () | Qt::ItemIsEditable);
        ui->listWidgetBgmnPresets->addItem(it);

    }
}
