/***************************************************************************
                          prefpagebgmnfavorites.cpp  -  description
                             -------------------
    begin                : Sun Aug 06 11:00:00 CEST 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefpagebgmnfavorites.h"
#include "../xrdcustomplot.h"
#include "../libXrdIO/structs.h"
#include "ui_prefpagebgmnfavorites.h"

PrefPageBGMNFavorites::PrefPageBGMNFavorites(QWidget *parent) :
    PrefPageTemplate(parent),
    ui(new Ui::PrefPageBGMNFavorites)
{
    ui->setupUi(this);

    initUi();
    initSettings();

    setStrDir();
}

PrefPageBGMNFavorites::~PrefPageBGMNFavorites()
{
    delete ui;
}


void PrefPageBGMNFavorites::initUi()
{
    refStrManager = BgmnRefStructureManager::getInstance();

    ui->treeWidget->header()->setContextMenuPolicy(Qt::CustomContextMenu);

    QString xa = "Diffraction Angle [" + QString(global::degree) + "2" + QString(global::theta) + "]";

    QStringList header;
    header << "File Name" << "Phase" << "Comment";
    ui->treeWidget->setColumnCount(header.size());
    ui->treeWidget->setHeaderLabels(header);
    ui->plotWidget->xAxis->setLabel(xa);
    ui->plotWidget->yAxis->setLabel("Intensity [%]");

    contextMenuHeader = new QMenu(this);

    for (int i = 1; i < ui->treeWidget->header()->count(); ++i) {
        QAction *act = new QAction(ui->treeWidget->headerItem()->text(i), contextMenuHeader);
        act->setCheckable(true);
        act->setData(i);
        contextMenuHeader->addAction(act);
    }

    connect(contextMenuHeader, SIGNAL(triggered(QAction*)), this, SLOT(toggleColumnHidden(QAction*)));
    connect(ui->treeWidget->header(), SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(customMenuRequested(QPoint)));
    connect(ui->treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(currentChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
}

void PrefPageBGMNFavorites::initSettings()
{
    QByteArray wdt = settings->value("bgmnProject/favorites/colWidth", QByteArray()).toByteArray();

    if (!wdt.isEmpty()) {
        ui->treeWidget->header()->restoreState(wdt);
    } else {
        QFontMetrics fm(ui->treeWidget->font());
        int w = fm.size(Qt::TextSingleLine, "Hydroxylapatite.str").width();
        ui->treeWidget->header()->resizeSection(0, 2*w);
        ui->treeWidget->header()->resizeSection(1, w);
        ui->treeWidget->header()->resizeSection(2, w/2);
    }

    sortCol = settings->value("bgmnProject/favorites/sortColumn", ui->treeWidget->sortColumn()).toInt();
    sortOrder = (Qt::SortOrder)settings->value("bgmnProject/favorites/sortOrder", 0).toInt();
    ui->treeWidget->sortItems(sortCol, sortOrder);
    ui->splitter->restoreState(settings->value("bgmnProject/favorites/splitter", QByteArray()).toByteArray());

    QList<QVariant> hiddenCols;
    hiddenCols.append(QVariant(2)); // hide the comment column by default
    hiddenCols = settings->value("bgmnProject/favorites/hiddenColumns", hiddenCols).toList();

    int n = qMin(ui->treeWidget->header()->count(), contextMenuHeader->actions().count() + 1);

    for (int i = 1; i < n; ++i) {
        bool b = hiddenCols.contains(QVariant(i));
        ui->treeWidget->setColumnHidden(i, b);
        contextMenuHeader->actions().at(i - 1)->setChecked(!b);
    }
}

void PrefPageBGMNFavorites::saveSettings()
{
    saveFavorites();

    settings->setValue("bgmnProject/favorites/sortColumn", ui->treeWidget->sortColumn());
    settings->setValue("bgmnProject/favorites/sortOrder", (int)ui->treeWidget->header()->sortIndicatorOrder());
    settings->setValue("bgmnProject/favorites/colWidth", ui->treeWidget->header()->saveState());
    settings->setValue("bgmnProject/favorites/splitter", ui->splitter->saveState());

    QList<QVariant> hiddenCols;
    for (int i = 1; i < ui->treeWidget->header()->count(); ++i) {
        if (ui->treeWidget->isColumnHidden(i)) hiddenCols.append(QVariant(i));
    }

    settings->setValue("bgmnProject/favorites/hiddenColumns", hiddenCols);
}

void PrefPageBGMNFavorites::customMenuRequested(QPoint p)
{
    contextMenuHeader->popup(ui->treeWidget->header()->viewport()->mapToGlobal(p));
}

void PrefPageBGMNFavorites::setStrDir()
{
    QStringList repos = settings->value("bgmnProject/structureDatabase", QStringList()).toStringList();
    refStrManager->createTree(ui->treeWidget, repos, false, Qt::ItemFlags(Qt::ItemIsAutoTristate | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled));
    setFavorites();

    ui->treeWidget->sortItems(0, Qt::AscendingOrder);

    for (int i = 0; i < ui->treeWidget->topLevelItemCount(); ++i) {
        ui->treeWidget->topLevelItem(i)->setExpanded(true);
    }

    for (int i = 0; i < ui->treeWidget->columnCount(); ++i) {
        if (ui->treeWidget->columnWidth(i) == 0) {
            ui->treeWidget->resizeColumnToContents(i);
        }
    }
}

void PrefPageBGMNFavorites::setFavorites()
{
    QSet<QString> favs = refStrManager->getFavFileNames();
    QTreeWidgetItemIterator itCheck(ui->treeWidget);

    while (*itCheck) {
        if (favs.contains((*itCheck)->data(0, Qt::UserRole).toString())) {
            (*itCheck)->setCheckState(0, Qt::Checked);
        }
        ++itCheck;
    }

    // don't know how to reset the iterator, so creating a new one for the second cycle
    QTreeWidgetItemIterator itExpand(ui->treeWidget);

    while (*itExpand) {
        if ((*itExpand)->type() == TYPE_DIRECTORY) {
            if ((*itExpand)->checkState(0) == Qt::Unchecked) {
                (*itExpand)->setExpanded(false);
            } else {
                (*itExpand)->setExpanded(true);
            }
        }
        ++itExpand;
    }
}

void PrefPageBGMNFavorites::saveFavorites()
{
    if (!ui->treeWidget->topLevelItemCount()) return;

    QStringList favs;
    QTreeWidgetItemIterator it(ui->treeWidget);

    while (*it) {
        if ((*it)->type() == TYPE_FILE) {
            if ((*it)->checkState(0) == Qt::Checked) {
                favs.append((*it)->data(0, Qt::UserRole).toString());
            }
        }
        ++it;
    }

    refStrManager->setFavorites(favs);
}

void PrefPageBGMNFavorites::currentChanged(QTreeWidgetItem *cur, QTreeWidgetItem *)
{
    ui->plotWidget->clearGraphs();

    if (!cur) {
        ui->plotWidget->replot();
        return;
    }

    QString f(cur->data(0, Qt::UserRole).toString());
    QVector<QVector<double> > v(refStrManager->getReferencePattern(f));

    if (v.size() < 2) {
        ui->plotWidget->replot();
        return;
    } else {
        if ((v.at(0).size() < 2) || (v.at(1).size() < 2)) {
            ui->plotWidget->replot();
            return;
        }
    }

    ui->plotWidget->addGraph();
    ui->plotWidget->graph(0)->setData(v.at(0), v.at(1));

    ui->plotWidget->setXLimits(v.at(0).first(), v.at(0).last());
    ui->plotWidget->rescaleAxes(true);
    ui->plotWidget->yAxis->setRange(0.0, 100.0);
    ui->plotWidget->replot();
}

void PrefPageBGMNFavorites::toggleColumnHidden(QAction *a)
{
    int c = a->data().toInt();
    ui->treeWidget->setColumnHidden(c, !a->isChecked());
}
