/***************************************************************************
                          prefpagebgmngoals.cpp  -  description
                             -------------------
    begin                : Thu Nov 09 09:40:00 CET 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefpagebgmngoals.h"
#include "ui_prefpagebgmngoals.h"

#include <QRegularExpressionValidator>

PrefPageBgmnGoals::PrefPageBgmnGoals(QWidget *parent) :
    PrefPageTemplate(parent),
    ui(new Ui::PrefPageBgmnGoals)
{
    ui->setupUi(this);
    initUi();
    validateInput();
    initSettings();
}

PrefPageBgmnGoals::~PrefPageBgmnGoals()
{
    delete ui;
}

void PrefPageBgmnGoals::initUi()
{
    QRegularExpression rx("[a-zA-Z0-9]+");
    QRegularExpressionValidator *goalValidator = new QRegularExpressionValidator(rx, this);
    ui->lineEditQuantPrefixRelative->setValidator(goalValidator);
    ui->lineEditQuantPrefixAbsolute->setValidator(goalValidator);
    ui->lineEditQuantDenomRelative->setValidator(goalValidator);
    ui->lineEditQuantDenomAbsolute->setValidator(goalValidator);

    connect(ui->checkBoxBgmnManageGoals, SIGNAL(toggled(bool)), this, SLOT(toggleQuantManagement(bool)));
    connect(ui->comboBoxQuantGoalsFormat, SIGNAL(currentIndexChanged(int)), this, SLOT(toggleQuantFormat(int)));
    connect(ui->lineEditQuantPrefixRelative, SIGNAL(textEdited(QString)), this, SLOT(validateInput()));
    connect(ui->lineEditQuantPrefixAbsolute, SIGNAL(textEdited(QString)), this, SLOT(validateInput()));
    connect(ui->lineEditQuantDenomRelative, SIGNAL(textEdited(QString)), this, SLOT(validateInput()));
    connect(ui->lineEditQuantDenomAbsolute, SIGNAL(textEdited(QString)), this, SLOT(validateInput()));
}

void PrefPageBgmnGoals::initSettings()
{
    ui->comboBoxQuantGoalsFormat->addItem(tr("Simple"));
    ui->comboBoxQuantGoalsFormat->addItem(tr("Extended"));

    ui->checkBoxBgmnManageGoals->setChecked(settings->value("bgmnProject/manageGoals", true).toBool());
    ui->comboBoxQuantGoalsFormat->setCurrentIndex(settings->value("bgmnProject/quantGoalFormat", 0).toInt());
    ui->lineEditQuantPrefixRelative->setText(settings->value("bgmnProject/quantGoalRelPrefix", "Q").toString());
    ui->lineEditQuantPrefixAbsolute->setText(settings->value("bgmnProject/quantGoalAbsPrefix", "Qabs").toString());
    ui->lineEditQuantDenomRelative->setText(settings->value("bgmnProject/quantSumRelPrefix", "sum").toString());
    ui->lineEditQuantDenomAbsolute->setText(settings->value("bgmnProject/quantSumAbsPrefix", "sumabs").toString());
    ui->checkBoxCalcBothQuantities->setChecked(settings->value("bgmnProject/quantGoalsBoth", false).toBool());

    toggleQuantManagement(ui->checkBoxBgmnManageGoals->isChecked());
    validateInput();
}

void PrefPageBgmnGoals::saveSettings()
{
    settings->setValue("bgmnProject/manageGoals", ui->checkBoxBgmnManageGoals->isChecked());
    settings->setValue("bgmnProject/quantGoalFormat", ui->comboBoxQuantGoalsFormat->currentIndex());
    settings->setValue("bgmnProject/quantGoalRelPrefix", ui->lineEditQuantPrefixRelative->text());
    settings->setValue("bgmnProject/quantGoalAbsPrefix", ui->lineEditQuantPrefixAbsolute->text());
    settings->setValue("bgmnProject/quantSumRelPrefix", ui->lineEditQuantDenomRelative->text());
    settings->setValue("bgmnProject/quantSumAbsPrefix", ui->lineEditQuantDenomAbsolute->text());
    settings->setValue("bgmnProject/quantGoalsBoth", ui->checkBoxCalcBothQuantities->isChecked());
}


void PrefPageBgmnGoals::toggleQuantManagement(bool b)
{
    ui->comboBoxQuantGoalsFormat->setEnabled(b);

    if (b) {
        bool g = ui->comboBoxQuantGoalsFormat->currentIndex() > 0 ? true : false;
        ui->groupBoxQuantGoalsRelative->setEnabled(g);
        ui->groupBoxQuantGoalsAbsolute->setEnabled(g);
    } else {
        ui->groupBoxQuantGoalsRelative->setEnabled(false);
        ui->groupBoxQuantGoalsAbsolute->setEnabled(false);
    }
}

void PrefPageBgmnGoals::toggleQuantFormat(int i)
{
    bool g = i > 0 ? true : false;
    ui->groupBoxQuantGoalsRelative->setEnabled(g);
    ui->groupBoxQuantGoalsAbsolute->setEnabled(g);
}

void PrefPageBgmnGoals::validateInput()
{
    QStringList warning;

    if (ui->lineEditQuantPrefixRelative->text() == ui->lineEditQuantPrefixAbsolute->text()) {
        warning << "Warning: Use different prefixes for relative and absolute quantity goals!";
    }

    if (ui->lineEditQuantDenomRelative->text() == ui->lineEditQuantDenomAbsolute->text()) {
        warning << "Warning: Use different denominator names for relative and absolute quantity goals!";
    }

    ui->labelInputWarning->setText(warning.join("\n"));
}
