/***************************************************************************
                          prefpagegeneral.cpp  -  description
                             -------------------
    begin                : Tue May 09 16:00:00 CEST 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QThread>
#include <QListWidget>
#include <QListWidgetItem>
#include "prefpagegeneral.h"
#include "ui_prefpagegeneral.h"

PrefPageGeneral::PrefPageGeneral(QWidget *parent) :
    PrefPageTemplate(parent),
    ui(new Ui::PrefPageGeneral)
{
    ui->setupUi(this);
    initUi();
    initSettings();
}

PrefPageGeneral::~PrefPageGeneral()
{
    delete ui;
}

void PrefPageGeneral::initUi()
{
    ui->comboBoxToolbarLayout->addItem(tr("Icons only"), QVariant("ToolButtonIconOnly"));
    ui->comboBoxToolbarLayout->addItem(tr("Text only"), QVariant("ToolButtonTextOnly"));
    ui->comboBoxToolbarLayout->addItem(tr("Text beside Icons"), QVariant("ToolButtonTextBesideIcon"));
    ui->comboBoxToolbarLayout->addItem(tr("Text under Icons"), QVariant("ToolButtonTextUnderIcon"));
    ui->comboBoxToolbarLayout->addItem(tr("Follow Style"), QVariant("ToolButtonFollowStyle"));

    ui->comboBoxIconTheme->addItem(tr("Light"), QVariant("profex-light"));
    ui->comboBoxIconTheme->addItem(tr("Light color"), QVariant("profex-light-colored"));
    ui->comboBoxIconTheme->addItem(tr("Dark"), QVariant("profex-dark"));

    ui->comboBoxDefaultProjectType->addItem(tr("FullProf"));
    ui->comboBoxDefaultProjectType->addItem(tr("BGMN"));

    ui->comboBoxNThreads->addItem("Automatic");
    ui->comboBoxNThreads->setItemData(0, 0);
    for (int i = 1; i <= QThread::idealThreadCount(); ++i) {
        ui->comboBoxNThreads->addItem(QString("%1").arg(i));
        ui->comboBoxNThreads->setItemData(i, i);
    }

    ui->comboBoxCheckUpdates->addItem("At every start", "s");
    ui->comboBoxCheckUpdates->addItem("Once a day", "d");
    ui->comboBoxCheckUpdates->addItem("Once a week", "w");
    ui->comboBoxCheckUpdates->addItem("Once a month", "m");
    ui->comboBoxCheckUpdates->addItem("Never", "n");
}


void PrefPageGeneral::initSettings()
{
    QStringList fpFext;
    QStringList bgmnFext;

    fpFext << "prf" << "pcr" << "sum";
    bgmnFext << "dia" << "sav" << "lst" << "par" << "val";

    ui->doubleSpinBoxDefaultWl->setValue(settings->defaultWavelength());
    ui->checkBoxDefaultWl->setChecked(settings->value("config/useDefaultWl", false).toBool());

    int pType = settings->value("config/defaultProjectType", 1).toInt();

    switch (pType) {
        case 0: ui->comboBoxDefaultProjectType->setCurrentIndex(0);
                break;
        case 1: ui->comboBoxDefaultProjectType->setCurrentIndex(1);
                break;
    }

    ui->checkBoxRestoreOpenProjects->setChecked(settings->value("config/restoreProjects", false).toBool());
    ui->comboBoxToolbarLayout->setCurrentIndex(settings->value("window/toolButtonLayout", Qt::ToolButtonIconOnly).toInt());
    ui->lineEditFpFext->setText(settings->value("fpProject/fileExtensions", fpFext).toStringList().join(" "));
    ui->lineEditBgmnFext->setText(settings->value("bgmnProject/fileExtensions", bgmnFext).toStringList().join(" "));
    ui->comboBoxNThreads->setCurrentIndex(settings->value("bgmnProject/nThreads", 0).toInt());
    ui->spinBoxParallelBatch->setValue(settings->value("config/batchParallelJobs", 1).toInt());

    QString iconMode = settings->value("config/iconTheme", QString()).toString();

    if (iconMode.isEmpty()) {
        if (settings->isDarkMode()) {
            // assuming dark gui mode
            iconMode = QString("profex-dark");
        } else {
            // assuming light gui mode
            iconMode = QString("profex-light-colored");
        }
    }

    if (iconMode == "profex-light")         ui->comboBoxIconTheme->setCurrentIndex(0);
    if (iconMode == "profex-light-colored") ui->comboBoxIconTheme->setCurrentIndex(1);
    if (iconMode == "profex-dark")          ui->comboBoxIconTheme->setCurrentIndex(2);

    QString upd = settings->value("config/updateFrequency", "n").toString();
    bool updSsl = settings->value("config/updateSsl", false).toBool();

    if (upd == "s") ui->comboBoxCheckUpdates->setCurrentIndex(0);
    if (upd == "d") ui->comboBoxCheckUpdates->setCurrentIndex(1);
    if (upd == "w") ui->comboBoxCheckUpdates->setCurrentIndex(2);
    if (upd == "m") ui->comboBoxCheckUpdates->setCurrentIndex(3);
    if (upd == "n") ui->comboBoxCheckUpdates->setCurrentIndex(4);
    ui->checkBoxUpdatesSsl->setChecked(updSsl);
}

void PrefPageGeneral::saveSettings()
{
    settings->setValue("config/defaultProjectType", ui->comboBoxDefaultProjectType->currentIndex());
    settings->setValue("config/defaultWl", ui->doubleSpinBoxDefaultWl->value());
    settings->setValue("config/useDefaultWl", ui->checkBoxDefaultWl->isChecked());
    settings->setValue("config/restoreProjects", ui->checkBoxRestoreOpenProjects->isChecked());
    settings->setValue("window/toolButtonLayout", ui->comboBoxToolbarLayout->currentIndex());
    settings->setValue("fpProject/fileExtensions", ui->lineEditFpFext->text().split(QRegularExpression("[\\s;,]+")));
    settings->setValue("bgmnProject/fileExtensions", ui->lineEditBgmnFext->text().split(QRegularExpression("[\\s;,]+")));
    settings->setValue("bgmnProject/nThreads", ui->comboBoxNThreads->itemData(ui->comboBoxNThreads->currentIndex()).toInt());
    settings->setValue("config/iconTheme", ui->comboBoxIconTheme->currentData(Qt::UserRole));
    settings->setValue("config/updateFrequency", ui->comboBoxCheckUpdates->currentData(Qt::UserRole).toString());
    settings->setValue("config/updateSsl", ui->checkBoxUpdatesSsl->isChecked());
    settings->setValue("config/batchParallelJobs", ui->spinBoxParallelBatch->value());
}
