/***************************************************************************
                          prefpagegraphs.h  -  description
                             -------------------
    begin                : Tue May 09 16:00:00 CEST 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefpagegraphs.h"
#include "ui_prefpagegraphs.h"

#include <QColorDialog>

PrefPageGraphs::PrefPageGraphs(QWidget *parent) :
    PrefPageTemplate(parent),
    ui(new Ui::PrefPageGraphs)
{
    ui->setupUi(this);

    initUi();
    initSettings();
}

PrefPageGraphs::~PrefPageGraphs()
{
    delete ui;
}

void PrefPageGraphs::initUi()
{
    ui->comboBoxMultiScans->addItem(tr("Show individual Scans"));
    ui->comboBoxMultiScans->addItem(tr("Sum Scans"));
    ui->comboBoxMultiScans->addItem(tr("Average Scans"));

    ui->comboBoxYaxisUnit->addItem(tr("Counts"));
    ui->comboBoxYaxisUnit->addItem(tr("Counts per second"));

    ui->comboBoxXaxisUnit->addItem(tr("° 2theta"));
    ui->comboBoxXaxisUnit->addItem(tr("d spacing"));

    ui->comboBoxYScaling->addItem(tr("Linear"));
    ui->comboBoxYScaling->addItem(tr("Square Root"));
    ui->comboBoxYScaling->addItem(tr("Log10"));

    ui->comboBoxActiveCompositionMode->addItem("Source Over");
    ui->comboBoxActiveCompositionMode->addItem("Destination Over");
    ui->comboBoxActiveCompositionMode->addItem("Clear");
    ui->comboBoxActiveCompositionMode->addItem("Source");
    ui->comboBoxActiveCompositionMode->addItem("Destination");
    ui->comboBoxActiveCompositionMode->addItem("Source In");
    ui->comboBoxActiveCompositionMode->addItem("Destination In");
    ui->comboBoxActiveCompositionMode->addItem("Source Out");
    ui->comboBoxActiveCompositionMode->addItem("Destination Out");
    ui->comboBoxActiveCompositionMode->addItem("Source Atop");
    ui->comboBoxActiveCompositionMode->addItem("Destination Atop");
    ui->comboBoxActiveCompositionMode->addItem("Xor");
    ui->comboBoxActiveCompositionMode->addItem("Plus");
    ui->comboBoxActiveCompositionMode->addItem("Multiply");
    ui->comboBoxActiveCompositionMode->addItem("Screen");
    ui->comboBoxActiveCompositionMode->addItem("Overlay");
    ui->comboBoxActiveCompositionMode->addItem("Darken");
    ui->comboBoxActiveCompositionMode->addItem("Lighten");
    ui->comboBoxActiveCompositionMode->addItem("Color Dodge");
    ui->comboBoxActiveCompositionMode->addItem("Color Burn");
    ui->comboBoxActiveCompositionMode->addItem("Hard Light");
    ui->comboBoxActiveCompositionMode->addItem("Soft Light");
    ui->comboBoxActiveCompositionMode->addItem("Difference");
    ui->comboBoxActiveCompositionMode->addItem("Exclusion");

    connect(ui->checkBoxBgColors, SIGNAL(toggled(bool)), ui->pushButtonBgColorIdle, SLOT(setEnabled(bool)));
    connect(ui->checkBoxBgColors, SIGNAL(toggled(bool)), ui->pushButtonBgColorActive, SLOT(setEnabled(bool)));
    connect(ui->checkBoxBgColors, SIGNAL(toggled(bool)), ui->pushButtonBgColorComplete, SLOT(setEnabled(bool)));

    connect(ui->pushButtonBgColorIdle,     SIGNAL(clicked(bool)), this, SLOT(graphIdleColorChanged()));
    connect(ui->pushButtonBgColorActive,   SIGNAL(clicked(bool)), this, SLOT(graphActiveColorChanged()));
    connect(ui->pushButtonBgColorComplete, SIGNAL(clicked(bool)), this, SLOT(graphCompleteColorChanged()));
}

void PrefPageGraphs::initSettings()
{
    ui->spinBoxDisplayLineWidth->setValue(settings->value("graph/lineWidth", 1).toInt());
    ui->checkBoxAntiAliasing->setChecked(settings->value("graph/antiAliasing", false).toBool());
    ui->checkBoxTungstenLines->setChecked(settings->value("graph/showTungstenSpecLines", false).toBool());
    ui->checkBoxCompleteFileName->setChecked(settings->value("graph/drawCompleteFileName", false).toBool());
    ui->spinBoxSymbolSize->setValue(settings->value("graph/crossSize", 1).toInt());

    ui->comboBoxXaxisUnit->setCurrentIndex(settings->value("graph/xAxisUnit", 0).toInt());
    ui->comboBoxYScaling->setCurrentIndex(settings->value("graph/yAxisScale", 0).toInt());
    ui->comboBoxYaxisUnit->setCurrentIndex(settings->value("graph/countsPerSecond", 0).toInt());

    ui->checkBoxMajorGrid->setChecked(settings->value("graph/showMajorGridLines", false).toBool());
    ui->checkBoxMinorGrid->setChecked(settings->value("graph/showMinorGridLines", false).toBool());
    ui->checkBoxFillActive->setChecked(settings->value("graph/fillActiveScan", true).toBool());

    ui->comboBoxMultiScans->setCurrentIndex(settings->value("scans/multiScanMode", 0).toInt());

    bool useBgColors = settings->value("graph/useBackgroundColors", true).toBool();

    ui->checkBoxBgColors->setChecked(useBgColors);
    ui->pushButtonBgColorIdle->setEnabled(useBgColors);
    ui->pushButtonBgColorActive->setEnabled(useBgColors);
    ui->pushButtonBgColorComplete->setEnabled(useBgColors);

    colBgIdle     = QColor(settings->value("graph/idleColor", "#ffffff").toString());
    colBgActive   = QColor(settings->value("graph/activeColor", "#ff9999").toString());
    colBgComplete = QColor(settings->value("graph/completeColor", "#99ff99").toString());

    ui->pushButtonBgColorIdle->setStyleSheet(QString("background-color:%1").arg(colBgIdle.name()));
    ui->pushButtonBgColorActive->setStyleSheet(QString("background-color:%1").arg(colBgActive.name()));
    ui->pushButtonBgColorComplete->setStyleSheet(QString("background-color:%1").arg(colBgComplete.name()));

    ui->checkBoxActiveScanLineWidth->setChecked(settings->value("graph/useActiveLineWidth", true).toBool());
    ui->spinBoxActiveScanLineWidth->setEnabled(settings->value("graph/useActiveLineWidth", true).toBool());
    ui->spinBoxActiveScanLineWidth->setValue(settings->value("graph/activeLineWidth", 2).toInt());

    ui->comboBoxActiveCompositionMode->setCurrentIndex(settings->value("graph/compositionMode", settings->isDarkMode() ? 12 : 13).toInt());
}

void PrefPageGraphs::saveSettings()
{
    settings->setValue("graph/lineWidth", ui->spinBoxDisplayLineWidth->value());
    settings->setValue("graph/antiAliasing", ui->checkBoxAntiAliasing->isChecked());
    settings->setValue("graph/crossSize", ui->spinBoxSymbolSize->value());

    settings->setValue("graph/xAxisUnit", ui->comboBoxXaxisUnit->currentIndex());
    settings->setValue("graph/yAxisScale", ui->comboBoxYScaling->currentIndex());
    settings->setValue("graph/countsPerSecond", ui->comboBoxYaxisUnit->currentIndex());
    settings->setValue("graph/fillActiveScan", ui->checkBoxFillActive->isChecked());

    settings->setValue("graph/showTungstenSpecLines", ui->checkBoxTungstenLines->isChecked());
    settings->setValue("graph/showMajorGridLines", ui->checkBoxMajorGrid->isChecked());
    settings->setValue("graph/showMinorGridLines", ui->checkBoxMinorGrid->isChecked());
    settings->setValue("graph/drawCompleteFileName", ui->checkBoxCompleteFileName->isChecked());
    settings->setValue("scans/multiScanMode", ui->comboBoxMultiScans->currentIndex());

    settings->setValue("graph/useBackgroundColors", ui->checkBoxBgColors->isChecked());

    settings->setValue("graph/idleColor", colBgIdle.name());
    settings->setValue("graph/activeColor", colBgActive.name());
    settings->setValue("graph/completeColor", colBgComplete.name());

    settings->setValue("graph/useActiveLineWidth", ui->checkBoxActiveScanLineWidth->isChecked());
    settings->setValue("graph/activeLineWidth", ui->spinBoxActiveScanLineWidth->value());
    settings->setValue("graph/compositionMode", ui->comboBoxActiveCompositionMode->currentIndex());
}


void PrefPageGraphs::graphIdleColorChanged()
{
    QColor tcolBgIdle = QColorDialog::getColor(colBgIdle, this, tr("Background Color for Idle Status"));

    if (tcolBgIdle.isValid()) {
        colBgIdle = tcolBgIdle;
        ui->pushButtonBgColorIdle->setStyleSheet(QString("background-color:%1").arg(colBgIdle.name()));
    }

}

void PrefPageGraphs::graphActiveColorChanged()
{
    QColor tcolBgActive = QColorDialog::getColor(colBgActive, this, tr("Background Color for Active Status"));

    if (tcolBgActive.isValid()) {
        colBgActive = tcolBgActive;
        ui->pushButtonBgColorActive->setStyleSheet(QString("background-color:%1").arg(colBgActive.name()));
    }
}

void PrefPageGraphs::graphCompleteColorChanged()
{
    QColor tcolBgComplete = QColorDialog::getColor(colBgComplete, this, tr("Background Color for Complete Status"));

    if (tcolBgComplete.isValid()) {
        colBgComplete = tcolBgComplete;
        ui->pushButtonBgColorComplete->setStyleSheet(QString("background-color:%1").arg(colBgComplete.name()));
    }
}
