/***************************************************************************
                          presetmenumanager.h  -  description
                             -------------------
    begin                : Fri Mar 19 18:29:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "presetmenumanager.h"
#include <QDirIterator>
#include <QDomDocument>
#include <QDebug>

PresetMenuManager::PresetMenuManager(QObject *parent) : QObject(parent)
{
    settings = SettingsManager::getInstance();
    menu = nullptr;
}

void PresetMenuManager::setMenu(QMenu *m)
{
    menu = m;
}

void PresetMenuManager::update()
{
    menu->clear();

    QMap<QString, QString> pfFiles = collectPresetFiles();
    QMap<QString, QString>::const_iterator it = pfFiles.constBegin();

    while (it != pfFiles.constEnd()) {
        QAction *act = new QAction(it.key(), nullptr);
        act->setData(QVariant(it.value()));
        connect(act, SIGNAL(triggered()), this, SLOT(presetActionTriggered()));
        menu->addAction(act);
        ++it;
    }
}

/*
 * collect all *.pfp files in the presets directory and subdirectories
 * returns a qmap with <presetName, absoluteFilePath>
 */
QMap<QString, QString> PresetMenuManager::collectPresetFiles()
{
    QMap<QString, QString> pfpFiles;
    QStringList bgmnPresets(settings->value("bgmnProject/presetDirectory", QStringList()).toStringList());

    if (bgmnPresets.isEmpty()) {
        qDebug() << QString("PresetMenuManager::collectPresetFiles(): No Preset repositories found");
        return pfpFiles;
    }

    for (int i = 0; i < bgmnPresets.size(); ++i) {
        QDirIterator it(bgmnPresets.at(i), QDirIterator::Subdirectories);

        while (it.hasNext()) {
            it.next();

            QFileInfo pfpfi(it.fileInfo());

            if (!pfpfi.isDir()) {
                if ((pfpfi.suffix() == "pfp") || (pfpfi.suffix() == "PFP")) {
                    // parse the preset name and append it to the list
                    QFile f(pfpfi.absoluteFilePath());

                    if (f.open(QFile::ReadOnly | QFile::Text)) {
                        QDomDocument doc("presetDoc");
                        doc.setContent(&f);
                        f.close();

                        QDomElement docElem = doc.documentElement();
                        QString name = docElem.attribute("name", QString());
                        QString vers = docElem.attribute("version", QString());

                        if (name.isEmpty()) {
                            name = pfpfi.baseName();
                        }

                        if (vers.isEmpty()) {
                            vers = QString();
                        } else {
                            vers = QString(" (version %1)").arg(vers); // prepare for creating a string "name (version)"
                        }

                        pfpFiles.insert(QString("%1%2").arg(name).arg(vers), pfpfi.absoluteFilePath());
                    }
                }
            }
        }
    }

    return pfpFiles;
}

void PresetMenuManager::presetActionTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (!action) return;

    emit sigApplyPreset(action->data().toString());
}
