/***************************************************************************
                          bgmnpresethandler.h  -  description
                             -------------------
    begin                : Sat Jan 07 10:30:00 CEST 2017
    copyright            : (C) 2017 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BGMNPRESETHANDLER_H
#define BGMNPRESETHANDLER_H

#include <QObject>
#include <QString>
#include <QByteArray>
#include <QCryptographicHash>
#include <QFileInfo>
#include <QDomElement>
#include <QDomDocument>
#include "../libXrdIO/structs.h"

using namespace global;

class BgmnPresetHandler : public QObject
{
public:
    BgmnPresetHandler(QObject *parent = 0);

    int createPreset();
    bool createPreset(const QString &cfile,
                      const QString &pfile,
                      const QString &cfileContent,
                      const QList<PeakRange> &pRanges,
                      const QList<global::CurveFitCurve> &cFitsCurves,
                      const QList<global::CurveFitVariable> &cFitVariables,
                      const QMap<QString, QVariant> &cFitControls,
                      const QList<global::AxObjectExcel> &axExcelControls,
                      const QList<BaseLineSnip> &baseLines);

    void parsePresetFile(const QString &);
    QString createProject(const QString &cfile, const QString &id, int nth);
    QList<PeakRange> getPeakIntegrals();
    bool hasBgmnProject();
    bool hasBaseLine();
    bool hasPeakIntegrals();
    bool hasCurveFit();
    bool hasAxExcelConfig();
    QMap<QString, QVariant> baseLineParameters();
    bool getCurveFits(QList<CurveFitCurve> &, QList<CurveFitVariable> &, QMap<QString, QVariant> &);
    QList<global::AxObjectExcel> getAxExcelConfig();

private:
    QStringList allTags;
    QStringList fileTags;
    QList<QDomElement> elements;
    QFileInfo presetFileInfo;

    bool createPresetDir(const QString &);
    QList<BgmnPresetFile> stringsToFileList(const QStringList &, const QFileInfo &);
    BgmnPresetFile writeTemplateFile(const QFileInfo &, const QString &);
    QList<BgmnPresetFile> copyFilesToTemplate(const QList<BgmnPresetFile> &, const QString &);
    FileType getFileType(const QFileInfo &, const QFileInfo &);
    QStringList allDevFiles(const QString &, const QFileInfo &, bool);

    QDomDocument createPresetXml(const QList<BgmnPresetFile> &,
                                 const QFileInfo &,
                                 const QList<PeakRange> &,
                                 const QList<CurveFitCurve> &,
                                 const QList<CurveFitVariable> &,
                                 const QMap<QString, QVariant> &,
                                 const QList<global::AxObjectExcel> &,
                                 const QList<BaseLineSnip> &);

    bool writePresetFile(const QDomDocument &, const QFileInfo &);
    QList<BgmnPresetFile> getFilesOfType(const QList<BgmnPresetFile> &, FileType);
    QList<PeakRange> domElementsToPeakIntegrals(const QList<QDomElement> &);
    int domElementsToBaseLineMode(const QList<QDomElement> &);
    QMap<QString, QVariant> domElementsToBaseLineParams(const QList<QDomElement> &);
    QList<BgmnPresetFile> domElementsToFiles(const QList<QDomElement> &);
    QList<CurveFitCurve> domElementsToCurveFitCurves(const QList<QDomElement> &);
    QList<CurveFitVariable> domElementsToCurveFitVariables(const QList<QDomElement> &);
    QMap<QString, QVariant> domElementsToCurveFitControls(const QList<QDomElement> &l);
    QList<global::AxObjectExcel> domElementsToAxExcel(const QList<QDomElement> &l);
    QList<BgmnPresetFile> toAbsolutePath(const QList<BgmnPresetFile> &, const QString &);
    QStringList checkExisting(const QList<BgmnPresetFile> &);
    QStringList checkMD5sum(const QList<BgmnPresetFile> &);
    QStringList checkOverwrite(const QList<BgmnPresetFile> &, const QString &);
    void showFileMissingWarning(const QStringList &);
    bool showChecksumWarning(const QStringList &);
    bool showOverwriteWarning(const QStringList &);
    bool fileChecks(const QList<BgmnPresetFile> &, const QString &);
    void copyFilesToDestination(const QList<BgmnPresetFile> &, const QString &, const QList<FileType> &);
    QString toDestinationFilePath(const BgmnPresetFile &, const QString &);
};

#endif // BGMNPRESETHANDLER_H
