/***************************************************************************
                          bgmnprojectwidget.h  -  description
                             -------------------
    begin                : Mon Jan 30 11:00:00 CEST 2012
    copyright            : (C) 2012 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BGMNPROJECTWIDGET_H
#define BGMNPROJECTWIDGET_H

#include <QtCore>
#include <QFileInfoList>
#include "projectwidget.h"
#include "bgmnhandler.h"
#include "eflechhandler.h"
#include "bgmnaddremovedialog.h"
#include "strfilebatcheditdialog.h"
#include "bgmnpresethandler.h"
#include "bgmnrefstructuremanager.h"
#include "resultsTreeWidget/bgmnresultstreewidget.h"
#include "../libXrdIO/parser/bgmnlstparser.h"
#include "../libXrdIO/parser/bgmnsavparser.h"
#include "../libXrdIO/parser/bgmnprotocolparser.h"
#include "filepoller/bgmndiahandler.h"
#include "filepoller/bgmndiahandler2.h"
#include "msoExport/msoexportexcel.h"

static const QString tempAuxString("temporaryPeakData");

class BgmnProjectWidget : public ProjectWidget
{
    Q_OBJECT
public:
    explicit BgmnProjectWidget(QWidget *parent = nullptr);
    ~BgmnProjectWidget();

    void initSettings();
    QString type() {return "BGMN";}

    void load(const QString &txt, const QString &grph, const QString &uid);

    void abort();
    void runPeakDetection();
    void addRemovePhase();
    QStringList getQuantitiesCsv();
    QStringList getResultsCsv();
    bool isRunning();
    void autoGuessOutputFiles(bool);
    void openProjectStrFiles();
    void closeProjectStrFiles();
    QStringList getChemistry(bool header);
    void createPreset();
    void createZipArchive(const QString &);
    QFileInfoList getAllProjectFilesList();
    QFileInfoList getSharedInputProjectFilesList();
    bool gatherSharedInputFiles(const QString &, const QFileInfoList &, bool);
    void saveCifFiles(bool bgmnComplete, const QMap<QString, QVariant> &auxDataGlobal = QMap<QString, QVariant>());
    QList<phaseData> getCifData(const QMap<QString, QVariant> &auxDataGlobal = QMap<QString, QVariant>());
    void saveCastepCellFiles();
    void setInternalStandard();
    void unsetInternalStandard();
    void generateReport();
    void resetMarginColor();
    QString getSampleID();
    void toggleRefStrFavorites(bool);
    void setReferenceStructureFileList();
    QString getResultsFile();
    int applyPreset(const QString &, bool overwrite = true);
    int convertRawDataFile();
    QString getInstrumentConfigFile(bool copy);
    void editProjectStrFiles();
    void addAmorphousPeak();
    void exportToExcel();
    void editExcelExport();

    BgmnResultsTreeWidget *resultsTree;
    ResultsTreeWidget * getResultsTree() {return static_cast<ResultsTreeWidget*>(resultsTree);}

private:
    BgmnDiaHandler *diaHandler;
    BgmnHandler *bgmnHandler;
    EflechHandler *eflechHandler;
    BgmnLstParser lparser;
    BgmnProtocolParser protocolParser;
    BgmnAddRemoveDialog *arPhaseDialog;
    StrFileBatchEditDialog *strFileEditDlg;
    BgmnRefStructureManager *refStrManager;
    QStringList defaultBgmnLocalParameters;
    QTime processTimer;
    MsoExportExcel *excelExporter;

    void setHighlighting(ControlFileEdit *);
    void bgmnRunProcess();
    void bgmnAbortProcess();

    void eflechRunProcess();
    void eflechAbortProcess();

    void teilRunProcess();
    void teilAbortProcess();

    void displayReferenceStructure(const QString &s);
    QStringList getGlobalIncludeList();
    QStringList getLocalIncludeList();
    void compileSumFormula(const QMap<QString, CrystalStructure> &);
    QString defaultControlFile(const QString &geq);
    void getSampleDisplacements(double &e1, double &e2, double &e3);
    void matchStrongestPeak();
    void parseProjectFiles();
    BgmnSavParser getSavParser(bool *ok = Q_NULLPTR);
    void setRexpDenom(const BgmnSavParser &);
    void setCustomWavelengt(const QString &);
    void removeStrucFiles(BgmnSavParser &, const QStringList &, bool);
    void insertStrucFiles(BgmnSavParser &, const QStringList &, bool);
    bool getPresetName(QString &, QString &);
    bool checkFilesExist(QFileInfoList &, const QStringList &flist, const QString &dir = QString());
    bool checkFileExists(QFileInfoList &, const QString &file, const QString &dir = QString());
    QVector<Hkl> getReferenceHklLines(const QString &);
    QVector<QVector<double> > getReferenceXyPattern(const QString &);
    Scan getReferenceScan(const QString &);
    QStringList getProjectStrFiles();
    QString getAbsoluteFilePath(const QString &file);
    void clearGui();
    bool eflechSourceDevFile(QString &, QString &);
    bool eflechActiveScanToTemp(const QString &);
    QString eflechTestString();
    void eflechAppendHklData(EflechHandler *, bool replace);
    QString elapsedTimeFormatted(int);
    void updateResults();
    QString writeReport();
    void deleteReport();
    void clearFileNames();
    void setupSearchMatchWidget(bool instr = true);

    int applyPresetBgmn(BgmnPresetHandler &, bool);
    void applyPresetIntegrals(BgmnPresetHandler &);
    void applyPresetBaseLine(BgmnPresetHandler &);
    void applyPresetCurveFits(BgmnPresetHandler &);
    void applyPresetMsoExcelExport(BgmnPresetHandler &);

public slots:
    QStringList contextHelp();
    void runRefinement();
    void runSearchMatch();
    void setReferenceStructure(QString);

private slots:
    void bgmnPollOutput();
    void bgmnComplete(QUuid);

    void eflechPollOutput();
    void eflechComplete();

    void teilPollOutput();
    void teilComplete();

    void searchMatchStatus(int);

    void referencePosClicked(double, double, double);
    void applyEditProjectStrFiles();
    void setReferenceXY();
};

#endif // BGMNPROJECTWIDGET_H
