/***************************************************************************
                          bgmndiahandler2.cpp  -  description
                             -------------------
    begin                : Fri Sep 10 22:22:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "bgmndiahandler2.h"
#include <QDateTime>
#include <QDebug>

BgmnDiaHandler2::BgmnDiaHandler2(QObject *parent) : QObject(parent)
{
    settings = SettingsManager::getInstance();
    scanControl = nullptr;
}

void BgmnDiaHandler2::init(GraphDataController *g, const QString &s, int nLines)
{
    scanControl = g;
    diaFile = s;
    Q_UNUSED(nLines);

    if (QFile::exists(s)) QFile::remove(s);

    lastFileSize = -1;
    pollInterval = settings->value("config/diaSizePollIntervalMS", 10).toInt();
    constantSizePolls = settings->value("config/diaSizePollCounter", 3).toInt();

    qDebug() << QString("BgmnDiaHandler::init(): Monitoring file %1").arg(s);
    qDebug() << QString("BgmnDiaHandler::init(): Size poll interval %1 ms").arg(pollInterval);
}

void BgmnDiaHandler2::reload()
{
    reload(diaFile);
}

void BgmnDiaHandler2::reload(const QString &s)
{
    if (!scanControl || s.isEmpty()) {
        qDebug() << QString("BgmnDiaHandler2::reload(): No diafile or scancontrol");
        return;
    }

    diaFile = s;
    currentSizePolls = 0;
    pollTimer.start(pollInterval, this);
}

void BgmnDiaHandler2::stop()
{
    if (pollTimer.isActive()) pollTimer.stop();
}

void BgmnDiaHandler2::timerEvent(QTimerEvent *)
{
    QFile fd(diaFile);
    if (!fd.exists()) return;
    qint64 curSize = fd.size();

    if (curSize == lastFileSize) {
        ++currentSizePolls;

        if (currentSizePolls >= constantSizePolls) {
            pollTimer.stop();
            reloadDelayed();
        }
    } else {
        lastFileSize = curSize;
    }
}

void BgmnDiaHandler2::reloadDelayed()
{
    if (!QFile::exists(diaFile)) {
        return;
    }

    int loaded = -1;
    int linesRead, linesTotal;

    if (scanControl->fileName() == diaFile) {
        loaded = scanControl->reloadDiaFile(true, false, linesRead, linesTotal);
    } else {
        loaded = scanControl->loadDiaFile(diaFile, true, scanControl->getSampleId(), false, linesRead, linesTotal);
    }

    if (loaded > 0) {
        scanControl->updateViews();
    }
}

