/***************************************************************************
                          graphdatacontroller.h  -  description
                             -------------------
    begin                : Tue Feb 11 18:00:00 CEST 2020
    copyright            : (C) 2020 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHDATACONTROLLER_H
#define GRAPHDATACONTROLLER_H

#include "abstractgraphview.h"
#include "graphdatamodel.h"
#include <QString>

class AbstractGraphView;

class GraphDataController : public QObject
{
    Q_OBJECT
public:
    explicit GraphDataController(GraphDataModel *m, QObject *parent = nullptr);

    void addView(AbstractGraphView *);

    void setViewStatus(int);

    int loadScanFile(const QString &s, const QString &uid, bool min, const QString &sid, bool upd = true);
    int loadDiaFile(const QString &s, bool min, const QString &sid, bool upd, int &ll, int &tl);
    bool saveScanFile(const QString &f, const QString &uid, int n, const QMap<QString, QVariant> &);
    int addScanFile(const QString &s, const QString &uid, bool min, bool upd = true);
    void appendScan(const Scan &s, bool upd = true);
    void appendHklScan(const Scan &s, bool upd = true);
    void replaceScan(int n, const Scan &s, bool upd = true);
    void replaceHklScan(int n, const Scan &s, bool upd = true);
    void removeScan(int, bool upd = true);
    void removeScan(const QUuid &, bool upd = true);
    void removeScans(const QList<QUuid> &);
    int reloadScanFile(bool min, bool upd);
    int reloadDiaFile(bool min, bool upd, int &ll, int &dl);
    QString getSampleId() const;
    QString fileName() const;
    QFileInfo fileInfo() const;
    double getWaveLength(double);
    int count() const;
    int countFiles() const;
    bool hasData() const;

    Scan * getScan(int i);
    Scan * getScan(const QUuid &);
    Scan * getScan(const QString &);
    Scan * getFirst();
    Scan * getLast();
    const Scan * first();
    const Scan * last();
    const Scan * at(int);
    QStringList getAllScanNames() const;
    QStringList getXyScanNames() const;
    QStringList getHklScanNames() const;

    Scan * firstActiveScan();
    QVector<Scan *> activeScans();
    void setActiveScans(const QList<int> &);
    void setActiveScans(const QList<QUuid> &);
    QList<int> activeScanIncides() const;
    bool hasActiveScan() const;
    void clearActiveScans() const;

    int indexOf(const Scan &) const;
    int indexOf(const QUuid &) const;
    QUuid scanUid(int) const;

    void setAngularCorrections(double, double, double);
    double getAngularCorrEPS1() const;
    double getAngularCorrEPS2() const;
    double getAngularCorrEPS3() const;

    void getScanWaveLength(double &ka1, double &ka2, double &kb, double fallback);
    void getFixedWaveLength(double &ka1, double &ka2, double &kb, double fixwl);
    void setCustomWaveLength(const QMap<QString, double> &);
    void clearCustomWaveLength();
    const QMap<QString, double> * getCustomWaveLength();

    void normalizeHklScan(Scan *);

    bool sortScanUp(int, bool upd = true);
    bool sortScanDown(int, bool upd = true);

    double getXmin() const;
    double getXmax() const;
    double getYmin() const;
    double getYmax() const;

    void updateViews(const AbstractGraphView *sender = nullptr);
    void resetViews(const AbstractGraphView *sender = nullptr);

    void yOffsetUp();
    void yOffsetDown();
    void xOffsetRight();
    void xOffsetLeft();
    void resetOffset();

    bool isStacked() const;

    /* changes all scans flagged as temporary to permament */
    void keepTemporary();
    void keepTemporary(const QList<QUuid> &);

    /* deletes all scans flagged as temporary */
    void clearTemporary();

private:
    QList<AbstractGraphView *> graphViews;
    GraphDataModel *graphModel;

signals:
    void dataUpdated();
};

#endif // GRAPHDATACONTROLLER_H

