/***************************************************************************
                          peakfitwidget.h  -  description
                             -------------------
    begin                : Tue Nov 16 22:00:00 CEST 2019
    copyright            : (C) 2019 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PEAKFITWIDGET_H
#define PEAKFITWIDGET_H

#include <QWidget>
#include <QVector>
#include <QTreeWidgetItem>
#include "../libXrdIO/settingsmanager.h"
#include "projectWidget/graphWidget/abstractgraphview.h"
#include "projectWidget/graphWidget/graphwindow.h"
#include "../libXrdIO/scan.h"
#include "../libXrdIO/scanops.h"
#include "../libXrdIO/curveFitting/curvefittingmanager.h"
#include "../libXrdIO/curveFitting/curvehandler.h"

namespace Ui {
class PeakFitWidget;
}

class PeakFitWidget : public AbstractGraphView
{
    Q_OBJECT

public:
    explicit PeakFitWidget(GraphDataController *c, GraphWindow *v, QWidget *parent = nullptr);
    ~PeakFitWidget();

    void clearTemporary();

    QList<global::CurveFitCurve> getCurves();
    QList<global::CurveFitVariable> getVariables();
    QMap<QString, global::CurveFitVariable> getVariablesByName();
    QMap<QString, QVariant> getControls();

    void restorePreset(const QList<global::CurveFitCurve> &, const QList<global::CurveFitVariable> &, const QMap<QString, QVariant> &);
    QString getReport(const QString &);

public slots:
    void updateView();

private:
    Ui::PeakFitWidget *ui;
    GraphWindow *graphView;
    bool wasUsed;
    QStringList headers;
    QList<QUuid> currentScans;
    QList<GenericCurve *> curveList;
    CurveFittingManager cfitManager;
    CurveHandler cfitHandler;
    int curveNumber;

    void initSettings();
    void saveSettings();
    bool initSolver();

    void addCurve(GenericCurve *curve, const QList<double> &values);
    void restoreCurve(const global::CurveFitCurve &, int, int);

    void readInitialValues(QVector<double> &, QVector<double> &, QVector<double> &);
    void writeFittedValues(const QVector<double> &);

    void setCurves(const QList<global::CurveFitCurve> &);
    void setVariables(const QList<global::CurveFitVariable> &);
    void setControls(const QMap<QString, QVariant> &);

    void updateCurveValues();
    void updateCurveValues(const QTreeWidgetItem *);
    void updateAreas(const QMap<QUuid, QString> &);
    void appendAreaItem(QTreeWidgetItem *);

    QPointF uncorrectedPoint(const QPointF &);

    const Scan * getActiveDataScan();

private slots:
    void fit();
    void append();
    void removeCurve();
    void keepTemporary();
    void toggleSelectRangeMode(bool);
    void toggleAddPeakMode(bool);
    void togglePeakFunction(int);
    void peakPreviewPoints(QPointF, QPointF, PeakPreviewMode);
    void rangePoints(QPointF, QPointF);
    void functionDoubleClicked(QTreeWidgetItem *, int);
    void variableDoubleClicked(QTreeWidgetItem *, int);
    void updateCurves();
    void clearAll();
    void undoControls();
    void cursorMessage(QString);
    void activeCurveChanged(QTreeWidgetItem*, int);

signals:
    void updateScans();
    void protocol(QString);
    void applyToAll();
};

#endif // PEAKFITWIDGET_H
