/***************************************************************************
                          projectsearchmatchwidget.h  -  description
                             -------------------
    begin                : Tue Jan 22 21:00:00 CEST 2019
    copyright            : (C) 2019 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PEAKLISTWIDGET_H
#define PEAKLISTWIDGET_H

#include <QWidget>
#include <QTreeWidgetItem>
#include <QStandardItemModel>
#include "graphWidget/abstractgraphview.h"
#include "bgmnrefstructuremanager.h"

namespace Ui {
class PeakListWidget;
}

class HklItemModel : public QStandardItemModel
{
    Q_OBJECT
public:
    explicit HklItemModel(const QUuid &, const QString &, QObject *parent = nullptr);

    inline QUuid const uid() {return _uid;}
    inline QString const name() {return _name;}

private:
    QUuid _uid;
    QString _name;
};

class PeakListWidget : public AbstractGraphView
{
    Q_OBJECT

public:
    explicit PeakListWidget(GraphDataController *c, QWidget *parent = nullptr);
    ~PeakListWidget();

    void clearAll();
    QUuid currentScanUid();

public slots:
    void updateView();
    void updateData();

private:
    Ui::PeakListWidget *ui;
    BgmnRefStructureManager *refStrManager;
    QMap<QUuid, HklItemModel*> hklModelStack;
    QStringList hklHeaderLabels;
    double waveLength;
    double twoThetaMin, twoThetaMax;
    QUuid allPeaksId;
    HklItemModel *allPeaksModel;
    QHash<QString, int> columns;
    QMenu *contextMenuHeader;
    bool saveSettingsRequested;
    int prevNumberOfScans;

    void initSettings();
    void saveSettings();
    void setCurrentModel(const QUuid &);
    void updateList();
    void addHklData(const Scan *);
    void setBoundariesTwoTheta(double tmin, double tmax, double wl);

    HklItemModel * dataToModel(const Scan *, bool normalize);
    void appendDataToModel(HklItemModel *, const Scan *, bool normalize);
    void appendHklToModel(HklItemModel *, const Hkl &, const Scan *, double normFactor);
    void removeDataFromModel(HklItemModel *, const QUuid &);
    void removeHklFromModel(HklItemModel *, const QUuid &);

    double getMaxHklIntensity(const Scan *);
    void filterScans(Scan &ref, Scan &act);
    Scan convoluteSampleFunction(const Scan &);
    void saveToCsv(const QString &);
    void saveToQualxD(const QString &);
    void saveToQualxT(const QString &);
    QList<int> hklToInt(const QString &);
    int indexOfHkl(const Scan *, const QUuid &);

private slots:
    void resultSelected(QTreeWidgetItem *);
    void showHklList(int);
    void saveHklList();
    void clearHklSelection();
    void selectHkl(const QItemSelection &, const QItemSelection &);
    void removeCurrentHkl();
    void customContextMenuRequested(const QPoint &);
    void toggleColumnHidden(QAction *);
    void headerResized();

signals:
    void resultsSelectionChanged(const QString &);
};

#endif // PEAKLISTWIDGET_H
