/***************************************************************************
                          scanlistwidget.h  -  description
                             -------------------
    begin                : Thu May 12 15:27:00 CEST 2016
    copyright            : (C) 2016 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCANLISTWIDGET_H
#define SCANLISTWIDGET_H

#include <QTreeWidget>
#include "graphWidget/abstractgraphview.h"

static const QString scanActiveText("A");

class ScanListWidget : public AbstractGraphView
{
    Q_OBJECT

public:
    ScanListWidget(GraphDataController *, QWidget *parent = 0);
    ~ScanListWidget();

    void updateView();
    void adjustHeaderSize();
    void toggleSubScans(bool);
    int currentIndex();

private:
    QTreeWidget *treeWidget;
    QMenu *slContextMenu;
    bool headerStateRestored;

    QMap<QUuid, Qt::CheckState> adjustTree();
    void generateTreeItems(const QMap<QUuid, Qt::CheckState> &, bool);
    void exportScan(Scan &);
    void copyScan(Scan &);

private slots:
    void initSettings();
    void saveSettings();
    void initConnections();
    void showContextMenu(const QPoint &);
    void exportSingleScan();
    void exportSingleScanWithAngularCorrections();
    void removeScan();
    void copySingleScan();
    void copySingleScanWithAngularCorrections();
    void copyHklData();
    void sortScanUp();
    void sortScanDown();
    void activeScanChanged(QTreeWidgetItem*, int);
    void scanParameterChanged(QTreeWidgetItem*, int);
    void changeScanVisibility(Scan *, int);
    void changeScanScaling(Scan *, const QString &);
    void changeScanYOffset(Scan *, const QString &);
    void changeScanXOffset(Scan *, const QString &);
    void addNoise();
};

#endif // SCANLISTWIDGET_H
