/***************************************************************************
                          projectstreewidget.cpp  -  description
                             -------------------
    begin                : Fri Oct 12 12:00:00 CEST 2018
    copyright            : (C) 2018 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "projectstreewidget.h"
#include <QDebug>

ProjectsTreeWidget::ProjectsTreeWidget(QWidget *parent) : QTreeWidget(parent)
{
    bgDefaultBrush = QTreeWidgetItem().background(0);
    bgHighlighBrush = QBrush(QColor("#ddddff"));
    setSortingEnabled(true);
    connect(this, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(highlightItems(QTreeWidgetItem*,QTreeWidgetItem*)));
}

void ProjectsTreeWidget::expandAll()
{
    for (int i = 0; i < topLevelItemCount(); ++i) {
        topLevelItem(i)->setExpanded(true);
    }
}

void ProjectsTreeWidget::collapseAll()
{
    for (int i = 0; i < topLevelItemCount(); ++i) {
        topLevelItem(i)->setExpanded(false);
    }
}

void ProjectsTreeWidget::highlightItems(QTreeWidgetItem *cur, QTreeWidgetItem *prev)
{
    Q_UNUSED(prev);
    if (!cur) return;

    if (indexOfTopLevelItem(cur) < 0) {
        QTreeWidgetItem *tlitem = cur->parent();
        highlightChildItems(tlitem->indexOfChild(cur));
    } else {
        if (cur->isExpanded()) {
            highlightToplevelItems();
        }
    }
}

void ProjectsTreeWidget::highlightChildItems(int n)
{
    for (int i = 0; i < topLevelItemCount(); ++i) {
        for (int c = 0; c < topLevelItem(i)->childCount(); ++c) {
            if (c == n) {
                topLevelItem(i)->child(c)->setBackground(0, bgHighlighBrush);
                topLevelItem(i)->child(c)->setBackground(1, bgHighlighBrush);
            } else {
                topLevelItem(i)->child(c)->setBackground(0, bgDefaultBrush);
                topLevelItem(i)->child(c)->setBackground(1, bgDefaultBrush);
            }
        }
    }
}

void ProjectsTreeWidget::highlightToplevelItems()
{
    for (int i = 0; i < topLevelItemCount(); ++i) {
        for (int c = 0; c < topLevelItem(i)->childCount(); ++c) {
            topLevelItem(i)->child(c)->setBackground(0, bgDefaultBrush);
            topLevelItem(i)->child(c)->setBackground(1, bgDefaultBrush);
        }
    }
}
