/***************************************************************************
                          coddownloadmanager.cpp  -  description
                             -------------------
    begin                : Thu Jan 11 18:30:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "coddownloadmanager.h"
#include <QRegularExpression>
#include <QDebug>

CodDownloadManager::CodDownloadManager(QObject *parent) : QObject(parent)
{
    qDebug() << QString("CodDownloadManager: entering constructor");
    connect(&cifWebCtrl, SIGNAL(finished(QNetworkReply*)), this, SLOT(cifDownloaded(QNetworkReply*)));
    qDebug() << QString("CodDownloadManager: constructor complete");
}

void CodDownloadManager::setRecords(const QStringList &l)
{
    records = l;
    numberOfFiles = l.size();
    currentRecord = QString();
    cifDownloadedData.clear();
}

void CodDownloadManager::startCifDownload(const QString &s)
{
    codUrl = s;

    if (!records.size()) {
        qDebug() << QString("CodDownloadManager::startCifDownload(): No records selected for download, exiting.");
        return;
    }

    wasCancelled = false;

    currentRecord = records.takeFirst();
    QUrl url(QString("%1/%2.cif").arg(codUrl).arg(currentRecord));
    qDebug() << QString("CodDownloadManager::startCifDownload(): Downloading %1").arg(url.toString());
    QNetworkRequest request(url);
    cifWebCtrl.get(request);
}

void CodDownloadManager::cifDownloaded(QNetworkReply* pReply)
{
    emit progress(numberOfFiles - records.size() + 1);

    QByteArray dl = pReply->readAll();
    if (dl.contains("data_")) cifDownloadedData.append(CifFile(currentRecord, QString(dl)));
    pReply->deleteLater();

    if (records.size()) {
        if (wasCancelled) {
            qDebug() << QString("CodDownloadManager::cifDownloaded(): Download was aborted by the user. Exiting.");
            emit cifDownloadComplete();
        } else {
            currentRecord = records.takeFirst();
            QUrl url(QString("%1/%2.cif").arg(codUrl).arg(currentRecord));
            qDebug() << QString("CodDownloadManager::cifDownloaded: downloading %1").arg(url.toString());
            QNetworkRequest request(url);
            cifWebCtrl.get(request);
        }
    } else {
        qDebug() << QString("CodDownloadManager::cifDownloaded: All downloaded");
        emit cifDownloadComplete();
    }
}

QList<CifFile> CodDownloadManager::getCifData() const
{
    return cifDownloadedData;
}

void CodDownloadManager::cancel()
{
    wasCancelled = true;
}
