/***************************************************************************
                          scantracerdialog.h  -  description
                             -------------------
    begin                : Wed May 12 19:00:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCANTRACERDIALOG_H
#define SCANTRACERDIALOG_H

#include <QDialog>
#include <QGraphicsItem>
#include <QTransform>
#include "scantracerscene.h"
#include "scantracerimagetracer.h"
#include "scantracercalibinfowidget.h"
#include "../libXrdIO/settingsmanager.h"

namespace Ui {
class ScanTracerDialog;
}

class ScanTracerDialog : public QDialog
{
    Q_OBJECT

public:
    explicit ScanTracerDialog(QWidget *parent = 0);
    ~ScanTracerDialog();

private:
    Ui::ScanTracerDialog *ui;
    SettingsManager *settings;
    ScanTracerScene *_scene;
    ScanTracerImageTracer *_tracer;

    QList<QPointF> _calibPointsPx;
    QList<QPointF> _calibPointsCalib;
    QTransform _pxToUnity;
    QTransform _unityToCoordinates;
    QPolygonF _traceCoord;
    QString _workingDir;
    QColor _scanColor;
    QColor _backgroundColor;

    void initSettings();
    void saveSettings();
    void blockSpinBoxSignals(bool);
    void updateCalibPointTR();
    QTransform getPxToUnity();
    QTransform getUnityToCoordinates();
    void addCalibPoint(const QPointF &);
    void loadImageFromDisk(const QString &);
    void setImageLimits();
    void clearCalibData();

    void resetGui();
    void resetCalibPage();
    void resetBaseLinePage();
    void resetTracePage();
    void resetSavePage();

private slots:
    void close();
    void loadImage();
    void pasteImage();
    void saveAs();
    void setCalibMode(bool);
    void setBaseLineMode(bool);
    void trace();
    void mouseClickPosition(QPointF);
    void mouseCoordinates(QPointF);
    void updateMatrix();
    void calibBLChanged();
    void calibBRChanged();
    void calibTLChanged();
    void calibCoordinatesChanged();
    void scanColorDialog();
    void backgroundColorDialog();
    void pickScanColor(bool);
    void pickBackgroundColor(bool);
    void nextPage();
    void scanColorPicked(QColor);
    void backgroundColorPicked(QColor);
    void colorSensitivity(int);
    void lineCenter(int);
};

#endif // SCANTRACERDIALOG_H
