/***************************************************************************
                          waterfallplotdialog.h  -  description
                             -------------------
    begin                : Mon Jul 26 18:26:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WATERFALLPLOTDIALOG_H
#define WATERFALLPLOTDIALOG_H

#include "../libXrdIO/settingsmanager.h"
#include "../libXrdIO/colorMaps/lutgenerator.h"
#include "../libXrdIO/scan.h"
#include "waterfallplotwidget.h"
#include <QDialog>
#include <QHash>
#include <QFileSystemWatcher>
#include <QTreeWidgetItem>

namespace Ui {
class WaterfallPlotDialog;
}

class WaterfallPlotDialog : public QDialog
{
    Q_OBJECT

public:
    explicit WaterfallPlotDialog(QWidget *parent = nullptr);
    ~WaterfallPlotDialog();

private:
    Ui::WaterfallPlotDialog *ui;
    SettingsManager *settings;
    QMenu *twContextMenu;
    QString workingDir;
    QString curFilter;
    QMap<QString, colorMaps::Lut> lutList;
    QFileSystemWatcher fsWatcher;
    QHash<QUuid, Scan> _scanHeap;
    QVector<QUuid> _uidList;
    QHash<QUuid, QString> _fileNames;
    int prevSelectionIdx;

    void initSettings();
    void saveSettings();
    void appendFileScans(const QString &file, const QString &filter);
    void updateYRangeSpinboxes();
    void removeNonexistentFiles();
    QString getLabel(const QString &);
    void sortItems();

private slots:
    void showContextMenu(const QPoint &);
    void close();
    void addFile();
    void removeFile();
    void removeFile(int);
    void clearScans();
    void updateScene();
    void selectDir();
    void monitorDir(bool);
    void updateLabels();
    void saveAs();
    void lutChanged();
    void colorTempChanged();
    void yScaleChanged();
    void xRangeChanged();
    void yRangeChanged();
    void mouseCoordinates(QString);
    void resetX();
    void resetY();
    void dirContentChanged(const QString &);
    void changeMonnitoringEnabled(QString);
    void itemsSorted();
    void updateCheckedItemList();
    void itemSelectionChanged();
    void itemClicked(QTreeWidgetItem*, int);
    void treeWidgetCheckAll();
    void treeWidgetUncheckAll();
    void treeWidgetCheckAllAbove();
    void treeWidgetUncheckAllAbove();
    void treeWidgetCheckAllBelow();
    void treeWidgetUncheckAllBelow();
    void updateXRangeSpinboxes();
};

#endif // WATERFALLPLOTDIALOG_H
