/***************************************************************************
                          waterfallplotwidget.h  -  description
                             -------------------
    begin                : Tue Aug 03 12:57:00 CEST 2021
    copyright            : (C) 2021 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WATERFALLPLOTWIDGET_H
#define WATERFALLPLOTWIDGET_H

#include <QFrame>
#include <QVector>
#include <QPainter>
#include <QMouseEvent>

#include "../libXrdIO/scan.h"
#include "../libXrdIO/colorMaps/lutgenerator.h"

class WaterfallPlotWidget : public QFrame
{
    Q_OBJECT
public:
    explicit WaterfallPlotWidget(QWidget *parent = nullptr);
    ~WaterfallPlotWidget();

    inline void setScanData(const QHash<QUuid, Scan> *s) {_scanHeap = s;}
    inline void setUidData(const QVector<QUuid> *v) {_uidList = v;}

    void updateScanData();
    void clearScans();
    void setLut(const colorMaps::Lut &);
    void setColorTemp(int);
    void setYscale(int);
    void setXrange(double, double);
    void setYrange(double, double);
    void getXrange(double &, double &);
    void getYrange(double &, double &);
    void getXlimits(double &, double &);
    void getYlimits(double &, double &);
    void resetXrange();
    void resetYrange();
    inline void blockUpdate(bool b) {_blockUpdate = b;}
    void save(const QString &, int, int);
    void toggleHighlightedScans(const QList<QUuid> &);

public slots:
    void updateAll();
    void setSmoothInterpolation(bool);
    void setDrawGridHorizontal(bool);
    void setDrawGridVertical(bool);

private:
    QImage _lineBuffer;
    QImage _scaledBuffer;
    bool _blockUpdate;
    int _nPoints;
    colorMaps::Lut _lut;
    bool _smoothInterpol;
    bool _drawGridH;
    bool _drawGridV;

    QStringList _labels;
    QRect _imageRect;
    QRect _fileNameRect;
    QRect _xaxisRect;
    QMap<int, int> _angToPixelMap;
    QVector<double> _majTicks;
    QVector<double> _minTicks;
    QList<QUuid> _hlIdx;
    const QHash<QUuid, Scan> *_scanHeap;
    const QVector<QUuid> *_uidList;

    double _clickPos;
    double _wmin;
    double _wmax;
    double _wminDisp;
    double _wmaxDisp;
    double _imin;
    double _imax;
    double _iminDisp;
    double _imaxDisp;
    double _wminDispClicked;
    double _wmaxDispClicked;

    double _iRangeMinLin;
    double _iRangeMaxLin;
    double _iRangeMinDisp;
    double _iRangeMaxDisp;

    int _yScale;
    int _lutOffset;
    int _lutSign;
    int _colTemp;

    void mouseMoveEvent(QMouseEvent *);
    void paintEvent(QPaintEvent *);
    void resizeEvent(QResizeEvent *);
    void wheelEvent(QWheelEvent *);
    void mousePressEvent(QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);

    int pixelToIndex(int, int, double, double, int, double, double);
    int angleToIndex(double);
    double pixelToAngle(int);
    void calcLimits();
    void calcXtickMarks();
    void calcDisplayYlimits();
    void calcDrawRegions(const QPaintDevice *canvas, const QFontMetrics &fm);
    void updateAngToPixelMap();
    void renderImageBuffer();
    void renderLine(const Scan &, QImage &, int y);
    void scaleDisplayBuffer();
    QRgb pixelValue(double);
    void setLutOffset();
    void zoomStep(int, int);

    void drawFileNames(QPainter &);
    void drawXaxis(QPainter &);
    void drawGrid(QPainter &);
    void drawHighlight(QPainter &);

    void renderPixmap(const QString &, int, int);
    void renderSvg(const QString &);
    void exportTextGrid(const QString &);

private slots:

signals:
    void sigMouseCoordinates(QString);
    void sigXrangeWasZoomed();

};

#endif // WATERFALLPLOTWIDGET_H
