/***************************************************************************
                          xrdcustomplot.cpp  -  description
                             -------------------
    begin                : Thu May 31 09:00:00 CEST 2018
    copyright            : (C) 2018 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xrdcustomplot.h"

XrdCustomPlot::XrdCustomPlot(QWidget *parent) : QCustomPlot(parent)
{
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    this->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);
    this->xAxis->axisRect()->setRangeDrag(Qt::Horizontal);
    this->xAxis->axisRect()->setRangeZoom(Qt::Horizontal);

    connect(this->xAxis, SIGNAL(rangeChanged(QCPRange)), this, SLOT(zoomX(QCPRange)));
    connect(this, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(zoomReset(QMouseEvent*)));
}

void XrdCustomPlot::zoomX(const QCPRange &newRange)
{
    double wmin = qMin(xmin, xmax);
    double wmax = qMax(xmin, xmax);

    QCPRange fixedRange(newRange);

    if (fixedRange.lower < wmin)
    {
        fixedRange.lower = wmin;
        fixedRange.upper = wmin + newRange.size();

        if (fixedRange.upper > wmax || qFuzzyCompare(newRange.size(), wmax-wmin)) {
            fixedRange.upper = wmax;
        }

        this->xAxis->setRange(fixedRange);
    } else if (fixedRange.upper > wmax)
    {
        fixedRange.upper = wmax;
        fixedRange.lower = wmax - newRange.size();

        if (fixedRange.lower < wmin || qFuzzyCompare(newRange.size(), wmax-wmin)) {
            fixedRange.lower = wmin;
        }

        this->xAxis->setRange(fixedRange);
    }

    this->replot();
}

void XrdCustomPlot::zoomReset(QMouseEvent *e)
{
    if (e->button() == Qt::RightButton) {
        this->xAxis->setRange(qMin(xmin, xmax), qMax(xmin, xmax));
        this->replot();
    }
}

void XrdCustomPlot::setXLimits(double mi, double ma)
{
    xmin = mi;
    xmax = ma;
}
